package reucd.gui;

import java.awt.Color;
import java.util.ArrayList;

import javax.swing.ImageIcon;
import javax.swing.JTree;
import javax.swing.ToolTipManager;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;

import reucd.main.Action;
import reucd.main.ActivityDiagram;
import reucd.main.ActivityEdge;
import reucd.main.ControlNode;

public class ActivityDiagramTree {
	
	private ArrayList edgesList;
	
	
	public ActivityDiagramTree() {
		
	}


	public JTree createTree(ArrayList internalEdges, ActivityDiagram activityDiagram) {
		
		edgesList = internalEdges;
		
		DefaultMutableTreeNode rootNode = new DefaultMutableTreeNode("Activity Diagram: " + activityDiagram.getName() );
		
		/** activity edges tree nodes creation */
		if (edgesList != null) {
			
			int i;
			for (i = 0; i < edgesList.size(); i++){
				
				ActivityEdge currentActivityEdge = ((ActivityEdge)edgesList.get(i));
				DefaultMutableTreeNode newEdge = new DefaultMutableTreeNode("Activity Edge");
				rootNode.add( newEdge );
				
				if (currentActivityEdge.getSourceType().equalsIgnoreCase(currentActivityEdge.TYPE_action)){
					
					Action sourceAction = ((Action)currentActivityEdge.getSourceAction());
					
					DefaultMutableTreeNode newSourceAction = new DefaultMutableTreeNode("Source -> Action ID: " + sourceAction.getActionID());
					newEdge.add( newSourceAction );
					
					DefaultMutableTreeNode newSourceActor = new DefaultMutableTreeNode("Actor: " + sourceAction.getPartition());
					newSourceAction.add( newSourceActor );
					
					DefaultMutableTreeNode newSourceDescription = new DefaultMutableTreeNode( getDescription(sourceAction));
					newSourceAction.add( newSourceDescription );
					
					if (currentActivityEdge.getDestinationType().equalsIgnoreCase(currentActivityEdge.TYPE_action)){
						Action destinationAction = ((Action)currentActivityEdge.getDestinationAction()); 
						
						DefaultMutableTreeNode newDestinationAction = new DefaultMutableTreeNode("Destination -> Action ID: " + destinationAction.getActionID());
						newEdge.add( newDestinationAction );
						
						DefaultMutableTreeNode newDestinationActor = new DefaultMutableTreeNode("Actor: " + destinationAction.getPartition());
						newDestinationAction.add( newDestinationActor );
						
						DefaultMutableTreeNode newDestinationDescription = new DefaultMutableTreeNode( getDescription(destinationAction));
						newDestinationAction.add( newDestinationDescription );
						
					}
					else if (currentActivityEdge.getDestinationType().equalsIgnoreCase(currentActivityEdge.TYPE_controlNode)){
						ControlNode destinationControlNode = ((ControlNode)currentActivityEdge.getDestinationControlNode());
						
						DefaultMutableTreeNode newDestinationControlNode = new DefaultMutableTreeNode("Destination -> " + destinationControlNode.getType() + " ID: " + destinationControlNode.getControlID());
						newEdge.add( newDestinationControlNode );
						
						
					}
					
				}
				else if (currentActivityEdge.getSourceType().equalsIgnoreCase(currentActivityEdge.TYPE_controlNode)){
					
					ControlNode sourceControlNode = ((ControlNode)currentActivityEdge.getSourceControlNode());
					
					DefaultMutableTreeNode newSourceControlNode = new DefaultMutableTreeNode("Source -> " + sourceControlNode.getType() + " ID: " + sourceControlNode.getControlID());
					newEdge.add( newSourceControlNode );
					
					
					if (currentActivityEdge.getDestinationType().equalsIgnoreCase(currentActivityEdge.TYPE_action)){
						Action destinationAction = ((Action)currentActivityEdge.getDestinationAction()); 
						
						DefaultMutableTreeNode newDestinationAction = new DefaultMutableTreeNode("Destination -> Action ID: " + destinationAction.getActionID());
						newEdge.add( newDestinationAction );
						
						DefaultMutableTreeNode newDestinationActor = new DefaultMutableTreeNode("Actor: " + destinationAction.getPartition());
						newDestinationAction.add( newDestinationActor );
						
						DefaultMutableTreeNode newDestinationDescription = new DefaultMutableTreeNode( getDescription(destinationAction));
						newDestinationAction.add( newDestinationDescription );
						
					}
					else if (currentActivityEdge.getDestinationType().equalsIgnoreCase(currentActivityEdge.TYPE_controlNode)){
						ControlNode destinationControlNode = ((ControlNode)currentActivityEdge.getDestinationControlNode());
						
						DefaultMutableTreeNode newDestinationControlNode = new DefaultMutableTreeNode("Destination -> " + destinationControlNode.getType() + " ID: " + destinationControlNode.getControlID());
						newEdge.add( newDestinationControlNode );
						
					}
				}
				
			}

		}
		
		JTree tree = new JTree(rootNode);
		
		/** enables tool tip mechanism */
		ToolTipManager.sharedInstance().registerComponent(tree);
		
		/** Set the icon for leaf nodes */
		ImageIcon leafIcon = new ImageIcon("leaf_node.gif");

		/** rendering tree nodes */		
		DefaultTreeCellRenderer renderer = new DefaultTreeCellRenderer();

		if (leafIcon != null) {			
			renderer.setLeafIcon(leafIcon);
		} else {
			/** Leaf icon missing, using default */
		}
		renderer.setBackground(Color.MAGENTA);
		renderer.setBackgroundNonSelectionColor(Color.WHITE);
		renderer.setToolTipText("activity edges");
		tree.setCellRenderer(renderer);
		
		return tree;
	}


	private String getDescription(Action sourceAction) {
		
		if (sourceAction.getInclude().length() > 0){
			return sourceAction.getInclude();
		}
		else if (sourceAction.getPerform().length() > 0){
			return sourceAction.getPerform();
		}
		else{
			return sourceAction.getDescription();
		}
		
	}

}
