package reucd.main;

public class Action {
	
	private String actor;
	private String description;
	private String include;
	private String perform;
	private String partition;
	private boolean decisionNodeCreated = false;
	private boolean forkNodeCreated = false;
	private boolean goesToFinalNode = false;
	private boolean isFake = false;
	private ControlNode decisionNode;
	private ControlNode forkNode;
	private ActivityDiagram owningActivityDiagram;
	private int actionID;


	public Action() {
		actor = "";
		description = "";
		include = "";
		perform = "";
		partition = "";
		actionID = -1;
	}
	
	
	
	public ActivityEdge createDecisionNodeForThisAction () {
		decisionNode = new ControlNode ();
		decisionNode.setOwnerAction(this);
		decisionNode.setType(decisionNode.TYPE_decision);
		decisionNode.setControlID(owningActivityDiagram.getNodeID());
		decisionNode.setOwningActivityDiagram(owningActivityDiagram);
		owningActivityDiagram.setNodeID(owningActivityDiagram.getNodeID() +1);
		ActivityEdge activityEdge = new ActivityEdge();
		activityEdge.setSourceAction(this);
		activityEdge.setSourceType(activityEdge.TYPE_action);
		activityEdge.setDestinationControlNode(decisionNode);
		activityEdge.setDestinationType(activityEdge.TYPE_controlNode);
		setDecisionNodeCreated(true);
		return activityEdge;
	}
	
	public ActivityEdge createForkNodeForThisAction() {
		forkNode = new ControlNode ();
		forkNode.setOwnerAction(this);
		forkNode.setType(forkNode.TYPE_fork);
		forkNode.setControlID(owningActivityDiagram.getNodeID());
		forkNode.setOwningActivityDiagram(owningActivityDiagram);
		owningActivityDiagram.setNodeID(owningActivityDiagram.getNodeID() +1);
		ActivityEdge activityEdge = new ActivityEdge();
		
		if (isDecisionNodeCreated()){
			activityEdge.setSourceControlNode(decisionNode);
			activityEdge.setSourceType(activityEdge.TYPE_controlNode);
			activityEdge.setDestinationControlNode(forkNode);
			activityEdge.setDestinationType(activityEdge.TYPE_controlNode);
			setForkNodeCreated(true);
			return activityEdge;
		}
		else{
			activityEdge.setSourceAction(this);
			activityEdge.setSourceType(activityEdge.TYPE_action);
			activityEdge.setDestinationControlNode(forkNode);
			activityEdge.setDestinationType(activityEdge.TYPE_controlNode);
			setForkNodeCreated(true);
			return activityEdge;
		}
		
		
	}
	
	
	
	
	public boolean isFake() {
		return isFake;
	}

	public void setFake(boolean isFake) {
		this.isFake = isFake;
	}


	public boolean isGoesToFinalNode() {
		return goesToFinalNode;
	}

	public void setGoesToFinalNode(boolean goesToFinalNode) {
		this.goesToFinalNode = goesToFinalNode;
	}

	public ControlNode getForkNode() {
		return forkNode;
	}

	public void setForkNode(ControlNode forkNode) {
		this.forkNode = forkNode;
	}

	public ControlNode getDecisionNode() {
		return decisionNode;
	}

	public void setDecisionNode(ControlNode decisionNode) {
		this.decisionNode = decisionNode;
	}

	public String getActor() {
		return actor;
	}

	public void setActor(String actor) {
		this.actor = actor;
	}


	public String getDescription() {
		return description;
	}


	public void setDescription(String description) {
		this.description = description;
	}


	public String getInclude() {
		return include;
	}


	public void setInclude(String include) {
		this.include = include;
	}


	public String getPartition() {
		return partition;
	}


	public void setPartition(String partition) {
		this.partition = partition;
	}


	public String getPerform() {
		return perform;
	}


	public void setPerform(String perform) {
		this.perform = perform;
	}

	public boolean isDecisionNodeCreated() {
		return decisionNodeCreated;
	}

	public void setDecisionNodeCreated(boolean decisionNodeCreated) {
		this.decisionNodeCreated = decisionNodeCreated;
	}
	
	public boolean isForkNodeCreated() {
		return forkNodeCreated;
	}

	public void setForkNodeCreated(boolean forkNodeCreated) {
		this.forkNodeCreated = forkNodeCreated;
	}

	public ActivityDiagram getOwningActivityDiagram() {
		return owningActivityDiagram;
	}

	public void setOwningActivityDiagram(ActivityDiagram owningActivityDiagram) {
		this.owningActivityDiagram = owningActivityDiagram;
	}

	public int getActionID() {
		return actionID;
	}

	public void setActionID(int actionID) {
		this.actionID = actionID;
	}

	
}
