package reucd.main;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Scanner;

import javax.swing.JFileChooser;

import org.apache.xerces.parsers.DOMParser;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import reucd.util.ExampleFileFilter;
import reucd.usecaseXML.*;

public class UseCaseDescriptions {

	public ArrayList useCases = null;
	public ArrayList actors = null;
	public ArrayList activityEdges = null;
	public ArrayList relationships = null;
	public ArrayList includeList = new ArrayList();
	public ArrayList extendList = new ArrayList();
	public ArrayList associationList = new ArrayList();
	public ArrayList actorGeneralizationList = new ArrayList();
	public ArrayList useCaseGeneralizationList = new ArrayList();
	public ArrayList performList = new ArrayList();
	
	ArrayList useCaseXMLNodesList = new ArrayList();
	ArrayList actorXMLNodesList = new ArrayList();
	ArrayList associationXMLNodesList = new ArrayList();
	
	/** xml document */
	private static Document doc = null;
	private Document nonStaticDoc = null;
	private static String   xmlStr = null;
	

	String useCaseIdPrefix = "_10_5_abd02bc_1136387958078_170256_";
	String actorIdPrefix = "_10_5_abd02bc_1136547759625_84118_";
	String associationIdPrefix ="_10_5_abd02bc_1136387971812_687526_";
	String generalizationIdPrefix ="_10_5_abd02bc_1136473338531_593297_";
	String includeIdPrefix ="_10_5_abd02bc_1137020931156_852431_";
	String extendIdPrefix ="_10_5_abd02bc_1136473301875_627168_";
	String extensionPointIdPrefix ="_10_5_abd02bc_1136473304140_607587_";
	String associationActorEndIdPrefix = "_10_5_abd02bc_1136387971812_339606_";
	String associationUseCaseEndIdPrefix = "_10_5_abd02bc_1136387971812_844672_";
	String textBoxIdPrefix = "_10_5_abd02bc_1137223812843_73455_";
	String conditionIdPrefix = "_10_5_abd02bc_1140434697593_151278_";
	String specificationIdPrefix = "_10_5_abd02bc_1140434697609_249774_";
	
	String useCaseType = "uml:UseCase";
	String actorType = "uml:Actor";
	String associationType = "uml:Association";
	String generalizationType = "uml:Generalization";
	String includeType = "uml:Include";
	String extendType = "uml:Extend";
	String extensionPointType = "uml:ExtensionPoint";
	/*those two are for conditions*/
	String conditionType = "uml:Constraint";
	String specificationType = "uml:OpaqueExpression";
	
	int useCaseCounter = 1;
	int actorCounter = 1;
	int associationCounter = 1;
	int generalizationCounter = 1;
	int includeCounter = 1;
	int extendCounter = 1;
	int extensionPointCounter = 1;
	int associationActorEndCounter = 1;
	int associationUseCaseEndCounter = 1;
	int textBoxCounter = 1; //textBox counter
	int weightCounter =  1;
	int conditionCounter = 1;
	int specificationCounter = 2;
	
	int actorLayoutSeparator = 100;
	int useCaseLayoutSeparator = 100;
	int seperationDistance = 100;


	//----constructor-------------
	public UseCaseDescriptions (){
		useCases = new ArrayList();
		actors = new ArrayList();
		activityEdges = new ArrayList();
		relationships = new ArrayList();
	}
	
	public void readFile(String fileName) {
		
		try {
			
			File file = new File(fileName);
			Scanner scanner = new Scanner(file);
			scanner.useDelimiter(System.getProperty("line.separator"));
			String newLine = "";
			boolean newLineReceived = false;
			
			
			while (scanner.hasNext()) {
				
				if (!newLineReceived ){
					newLine = scanner.next();
				}
				newLineReceived = false;

				//parsing by section
				try {
					if (newLine.contains("Actor:")){
						Actor actor = new Actor();
						newLine = processActor(scanner, actor);
						(getActors()).add(actor);
						newLineReceived = true;
					}				
					else if (newLine.contains("Use Case Name:")){
						UseCase useCase = new UseCase();
						NameSection nameSection = new NameSection();
						newLine = processNameSection (scanner, nameSection);
						useCase.setNameSection(nameSection);
						(getUseCases()).add(useCase);
						newLineReceived = true;
					}
					else if (newLine.contains("Basic Flow:")){
						BasicFlowSection basicFlowSection = new BasicFlowSection ();
						processBFSection (scanner, basicFlowSection);
						//getting the last use case object from the use case descriptions objects and
						//setting basicSectoin as its BasicSection component
						((UseCase)(((getUseCases()).get((getUseCases().size())-1))))
						.setBasicSection(basicFlowSection);
					}
					else if (newLine.contains("Alternative Flows:")){
						AlternativeFlowSection alternativeFlowSection = new AlternativeFlowSection();
						newLine = processAFSection (scanner,alternativeFlowSection);
						((UseCase)(((getUseCases()).get((getUseCases().size())-1))))
						.setAlternativeFlowSection(alternativeFlowSection);
						newLineReceived = true;
					}
					else if (newLine.contains("Subflows:")){
						SubflowSection subflowSection = new SubflowSection ();  
						newLine = processSFSection (scanner, subflowSection);
						((UseCase)(((getUseCases()).get((getUseCases().size())-1))))
						.setSubflowSection(subflowSection);
						newLineReceived = true;
					}
					else if (newLine.contains("Extension Points:")){
						ExtensionPointSection extensionPointSection = new ExtensionPointSection ();
						newLine = processEPSection (scanner, extensionPointSection);
						((UseCase)(((getUseCases()).get((getUseCases().size())-1))))
						.setExtensionPointsSection(extensionPointSection);
						newLineReceived = true;
					}
				} catch (StringIndexOutOfBoundsException e) {
					//do nothing
				}
				
			}
			scanner.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		
	}//end of readFile function
	
	private String processActor(Scanner scanner, Actor actor) {
		String actorLine ="";
		boolean actorProcessed = false;
		
		while  (!actorProcessed){
			actorLine = scanner.next();
			
			if (actorLine.contains("SPECIALIZES")){
				
				String specializeLine = actorLine;		
				specializeLine = specializeLine.replace("SPECIALIZES","");
				specializeLine = specializeLine.trim();
				actor.setSpecializesActor(specializeLine);
				actor.setSpecializing(true);
			}
			else if (actorLine.contains("ABSTRACT")){
				
				actor.setAbstract(true);
			}
			else if (actorLine.contains(":")){
				actorProcessed = true;
			}
			else if (actorLine.length()>0){//that's the actor name
				
				String actorName = actorLine;
				actorName = actorName.trim();
				actor.setName(actorName);
			}
		}
		
		return actorLine;
	}//end of processActor function


	private String processEPSection(Scanner scanner, ExtensionPointSection extensionPointSection) {
		//at this point we are at the "Extension Points:" label
		
		String newLine ="";
		
		while (scanner.hasNext()) {
			
			newLine = scanner.next();

			try {
				if (newLine.contains("PRIVATE")){
					(extensionPointSection.getExtensionPoints()).add(processPrEP(scanner, newLine)); 
				}
				else if (newLine.contains("PUBLIC")){
					//this returns one extensionPoint object
					(extensionPointSection.getExtensionPoints()).add(processPuEP(scanner, newLine)); 
					//there wil be two types:(a) declarations and (b) behavior definitions
				}
				else if (newLine.contains(":")){
					return newLine;
				}
			} catch (StringIndexOutOfBoundsException e) {
				//do nothing
			}		
		}
		return newLine;
	}


	private ExtensionPoint processPuEP(Scanner scanner, String publicEPLine) {
		//we are at a label that contains "PUBLIC"
		
		ExtensionPoint extensionPoint = new ExtensionPoint ();
		

		if (publicEPLine.contains("BEHAVIOR")){//its a behavior definition
			processPuEPBehavior(scanner, extensionPoint);
		}
		else if (publicEPLine.contains("EXTENSION")){//its just a declaration
			processPuEPDeclaration(scanner, extensionPoint);
		}		
		
		return extensionPoint;
	}


	private void processPuEPDeclaration(Scanner scanner, ExtensionPoint extensionPoint) {
		//here we are still at PUBLIC EXTENSION POINT declaration
		
		String declarationLine = "";
		extensionPoint.setExtensionPointType(extensionPoint.TYPE_publicDeclaration);

		while (scanner.hasNext()){
			declarationLine = scanner.next();
			if (declarationLine.length() > 0){ //its not an empty line
				declarationLine = declarationLine.trim();
				extensionPoint.setExtensionPointName(declarationLine);
				
				return;
			}
		}
	}


	private void processPuEPBehavior(Scanner scanner, ExtensionPoint extensionPoint) {
		//here we are still at PUBLIC EXTENSION POINT BEHAVIOR
		
		boolean PuEPHeaderProcessed = false;
		String behaviorLine = "";
		extensionPoint.setExtensionPointType(extensionPoint.TYPE_publicBehavior);

		while (!PuEPHeaderProcessed){

			if (behaviorLine.contains("EXTENDING")){ //The base use case name and header
				
				String baseUC;
				String baseUCExtensionPointName;
				Scanner extendingBaseHeaderScanner = new Scanner (behaviorLine);
				
				//getting the base use case name
				extendingBaseHeaderScanner.useDelimiter(":");
				baseUC = extendingBaseHeaderScanner.next();
				baseUC = baseUC.replace("EXTENDING","");
				baseUC = baseUC.replace("{","");
				baseUC = baseUC.trim();
				extensionPoint.setBaseUC(baseUC);
				
				//getting the base use case header
				extendingBaseHeaderScanner.useDelimiter("}");
				baseUCExtensionPointName = extendingBaseHeaderScanner.next();
				baseUCExtensionPointName = baseUCExtensionPointName.replace(":","");
				baseUCExtensionPointName = baseUCExtensionPointName.trim();
				extensionPoint.setExtensionPointName(baseUCExtensionPointName);
			}
			else if (behaviorLine.contains("FLOW")){
				extensionPoint.setAlternativeFlow(processAF (scanner, behaviorLine)); //process the AF part in the PrEP
			}
			else if (behaviorLine.contains("CONTINUE")){
				String continueHeaderName = behaviorLine;
				continueHeaderName = continueHeaderName.replace("CONTINUE", "");
				continueHeaderName = continueHeaderName.replace("{", "");
				continueHeaderName = continueHeaderName.replace("}", "");
				continueHeaderName  = continueHeaderName.trim();
				
				extensionPoint.setContinueHeaderName(continueHeaderName);
				
				
				PuEPHeaderProcessed = true;
			}
			
			if (!PuEPHeaderProcessed){
				behaviorLine = scanner.next();
			}
		}
		
	}


	private ExtensionPoint processPrEP(Scanner scanner, String PrEPLine) {
		
		boolean PrEPHeaderEnded = false;//keep going through the PrEP untill we reach the end
		ExtensionPoint extensionPoint = new ExtensionPoint();
		extensionPoint.setExtensionPointType(extensionPoint.TYPE_privatePoint);
		
		while (!PrEPHeaderEnded){
			if (PrEPLine.contains("PRIVATE")){
				//here we reach the PRIVATE EXTENSION POINT
				//we are not supposed to do anything here but continue on with the next line
			}
			else if (PrEPLine.contains("FLOW")){
				extensionPoint.setAlternativeFlow(processAF (scanner, PrEPLine)); //process the AF part in the PrEP
				
			}
			else if (PrEPLine.contains("CONTINUE")){
				String continueHeaderName = PrEPLine;
				continueHeaderName = continueHeaderName.replace("CONTINUE", "");
				continueHeaderName = continueHeaderName.replace("{", "");
				continueHeaderName = continueHeaderName.replace("}", "");
				continueHeaderName  = continueHeaderName.trim();
				
				extensionPoint.setContinueHeaderName(continueHeaderName);
				
				
				PrEPHeaderEnded = true;
			}
			
			
			if (!PrEPHeaderEnded){
				PrEPLine = scanner.next();
			}
		}
		
		return extensionPoint;
	}//end of processPrEP function


	private String processSFSection(Scanner scanner, SubflowSection subflowSection) {
		//right now we are at the Subflows: label
		
		String subflowLine = "";
		while (scanner.hasNext()){
			subflowLine = scanner.next();
			if (subflowLine.contains("SUBFLOW")){
				//add one subflow object to the subflowSection object
				(subflowSection.getSubflows()).add(processSF(scanner, subflowLine)); 
			}
			else if (subflowLine.contains(":")){
				return subflowLine;
			}
		}
		return subflowLine;
	}


	private Subflow processSF(Scanner scanner, String subflowLine) {
		//we are already at SUBFLOW BEGIN line
		
		boolean SFHeaderEnded = false;//keep going through the subflow untill we reach the end
		Subflow subflow = new Subflow ();
		
		while (!SFHeaderEnded){
			if (subflowLine.contains("SUBFLOW")){
				//get the subflow name from Begin

				String subflowName;
				
				subflowName = subflowLine;
				subflowName = subflowName.replace("SUBFLOW","");
				subflowName = subflowName.trim();
				subflow.setName(subflowName);
			}
			else if(subflowLine.contains("BEGIN")){
				//we have reached the header for the subflow
				subflow.setTopLevelHeader(processHeader(scanner,subflowLine, null));//returns the highest level header object
				SFHeaderEnded = true;
			}
			
			
			if (!SFHeaderEnded){
				subflowLine = scanner.next();//if we haven't reach the end of the subflow
											//then keep reading
			}
			
		}
		return subflow;
	}//end of processSF function


	private String processAFSection(Scanner scanner, AlternativeFlowSection alternativeFlowSection) {
		String newLine ="";
		
		while (scanner.hasNext()) {
			newLine = scanner.next();
			try {
				if (newLine.contains("FLOW")){//new alternative flow
					//adding one alternative flow to the alternative flow section
					(alternativeFlowSection.getAlternativeFlows()).add(processAF(scanner, newLine));	
				}
				else if (newLine.contains(":")){
					return newLine;
				}
				else if (newLine.contains("EXTENSION")){
					return newLine;
				}
			} catch (StringIndexOutOfBoundsException e) {
				//do nothing
			}		
		}
		return newLine;
	}


	private AlternativeFlow processAF(Scanner scanner, String newLine) {
		//we are already at the first AT line in the AF
		
		boolean AFHeaderStarted = false;//keep going through ATs ANDs and IFs till we reach a header
		boolean andSeperator = true;//makes sure there is no two straight ATs without an AND  
		AlternativeFlow alternativeFlow = new AlternativeFlow ();
		
		
		while (!AFHeaderStarted){
			
			if (newLine.contains("AT")){//an AT statement
				if (andSeperator){
					//this is just to get the new string back from the function call 
					StringWrapper newLineWrappedString = new StringWrapper (newLine); 
					(alternativeFlow.getAts()).add(processAt(scanner,newLineWrappedString)); //adding the AT
					newLine = newLineWrappedString.getTheContainedString();
				}
				andSeperator = false;
			}
			else if (newLine.contains("IF")){
				//IF statement
				alternativeFlow.setIfStatement(processIf(newLine));//setting the if statement
				if (scanner.hasNext()){
					newLine = scanner.next();
				}
			}
			else if (newLine.contains("AND")){
				//AND statement
				andSeperator = true;
				if (scanner.hasNext()){
					newLine = scanner.next();
				}
			}
			else if (newLine.contains("BEGIN")){
				//The AF header has started
				alternativeFlow.setTopLevelHeader(processHeader(scanner, newLine, null)); //setting the top level header
				AFHeaderStarted = true;
			}
			else if (newLine.contains("FLOW")){
				//The AF header has started
				newLine = newLine.replace("FLOW","");
				newLine = newLine.trim();
				alternativeFlow.setFlow(newLine);
				newLine = scanner.next();
			}
		}
		
		return alternativeFlow;
	}//end of processAF function


	private void processBFSection(Scanner scanner, BasicFlowSection basicFlowSection) {
		while (scanner.hasNext()) {
			String newLine = scanner.next();
			

			try {
				if (newLine.contains("{")){ //looking for the header {BEGIN Use Case}
					basicFlowSection.setTopLevelHeader(processHeader(scanner, newLine, null));
					//the basic flow only references the top level header
					//the top level header in turn will refernce its lower level headers
					return;//this only works because there is only one basic flow in the BF section
				}
			} catch (StringIndexOutOfBoundsException e) {
				//do nothing
			}		
		}
	}//end of processBFSection function
	
	
	
	private At processAt(Scanner scanner, StringWrapper newLineWrappedString) {
		//AT {header1 - low level}
		//Actor -> some action
		//Actor -> some action 2
		//AND
		//AT {header2 - high level) //include all its lower level headers
		//AND 
		//AT {header 3 -low level) //all of its actions
		
		At atObject = new At ();
		
		String atStatementHeader;
		String atLine = newLineWrappedString.getTheContainedString();
		Scanner atScanner = new Scanner (atLine);
		
		//right now we are at the AT line
		atScanner.useDelimiter("}");
		atStatementHeader = atScanner.next();
		atStatementHeader = atStatementHeader.replace("AT","");
		atStatementHeader = atStatementHeader.replace("{","");
		atStatementHeader = atStatementHeader.trim();
		atObject.setAtHeader(atStatementHeader);
		
		
		//time to look ahead to find out whats next; action, AND, or the IF
		boolean moreActions = true;
		String nextLineAfterAt = "";
		
		while (scanner.hasNext() && moreActions){
			nextLineAfterAt = scanner.next();
			if (nextLineAfterAt.contains("->")){
				(atObject.getActions()).add(processAction(nextLineAfterAt));
			}
			else if (nextLineAfterAt.contains("IF") ||
					nextLineAfterAt.contains("AND")||
					nextLineAfterAt.contains("BEGIN")){
				moreActions = false;
			}
		}
		
		newLineWrappedString.setTheContainedString(nextLineAfterAt);
		
		return atObject;
	}


	private String processNameSection(Scanner scanner, NameSection nameSection) {
		
		String useCaseNameLine = "";
		String useCaseName = "";
		String specializesUseCase = "";
		String implementesUseCase = "";
		boolean isSpecializing = false;
		boolean isImplementing= false;
		boolean isAbstract =  false;
		boolean nameSectionProcessed = false;
		
		try{
			while (!nameSectionProcessed){
				useCaseNameLine  = scanner.next();
				
				if (useCaseNameLine.contains("ABSTRACT")){
					isAbstract = true;
					nameSection.setAbstract(isAbstract);
					
				}
				else if (useCaseNameLine.contains("SPECIALIZES")){

					specializesUseCase = useCaseNameLine;
					specializesUseCase = specializesUseCase.replace("SPECIALIZES","");
					specializesUseCase = specializesUseCase.trim();
					isSpecializing = true;
					nameSection.setSpecializing(isSpecializing);
					nameSection.setSpecializesUseCase(specializesUseCase);
				}
				else if (useCaseNameLine.contains("IMPLEMENTS")){
					implementesUseCase = useCaseNameLine;
					implementesUseCase = implementesUseCase.replace("IMPLEMENTS","");
					implementesUseCase = implementesUseCase.trim();
					isImplementing = true;
					nameSection.setImplementing(isImplementing);
					nameSection.setImplementsUsecase(implementesUseCase);
					
				}
				else if (useCaseNameLine.contains(":")){
					nameSectionProcessed = true;
				}
				else if (useCaseNameLine.length()>0){//this is the use case name
					useCaseName = useCaseNameLine;
					useCaseName = useCaseName.trim();
					nameSection.setName(useCaseName);
				}
			}
		} catch (java.util.NoSuchElementException e){
			nameSectionProcessed = true;
			return useCaseNameLine;
		}
		
		
		return useCaseNameLine;
		
	}//end of processNameSection function
	
	
	private Action processAction(String anAction) {
		//This function returns only one single Action object
		
		Action action = new Action ();
		
		if (anAction.contains("->")){//here it is just a regular action statement
			
			String actor;
			String description;
			Scanner actionScanner = new Scanner(anAction);
			
			//actionScanner.next();//getting past the '' character
			
			//getting the actor
			actionScanner.useDelimiter("->");
			actor = actionScanner.next();
			actor = actor.replace(""," ");
			actor = actor.trim();
			action.setActor(actor);
			action.setPartition(actor);
			
			//getting the description
			description = actionScanner.next();
			description = description.trim();
			action.setDescription(description);
			
		}
		else if (anAction.contains("PERFORM")){//here it is perform statement
			
			String performSubflow;
			Scanner actionScanner = new Scanner(anAction);
			actionScanner.useDelimiter("PERFORM");
			actionScanner.next();//skipping till the PERFORM keyword
			performSubflow = actionScanner.next();//getting the resume statement
			performSubflow = performSubflow.trim();
			action.setPerform(performSubflow);
			action.setPartition("SYSTEM");
		}
		else if (anAction.contains("INCLUDE")){//here it is an include statement
			
			String inclusionUseCase;
			Scanner actionScanner = new Scanner(anAction);
			actionScanner.useDelimiter("INCLUDE");
			actionScanner.next();//skipping till the INCLUDE keyword
			inclusionUseCase = actionScanner.next();//getting the resume statement
			inclusionUseCase = inclusionUseCase.trim();
			action.setInclude(inclusionUseCase);
			action.setPartition("SYSTEM");
		}
		
		return action;
	}//end of processAction function
	
	
	private Header processHeader(Scanner scanner, String newLine, Header TLHeader) {
		
		boolean beginHeaderProcessed = false;
		boolean headerProcessCompleted = false;
		Header header = new Header ();
		header.setParentHeader(TLHeader);
		
		while (!headerProcessCompleted){//keep processing the header till you get to END
			Scanner headerScanner = new Scanner(newLine);
			
			try {
				//meanwhile look for actions
				if (newLine.contains("->") || newLine.contains("PERFORM") 
						|| newLine.contains("INCLUDE") ){
					//an action statement
					(header.getActions()).add(processAction(newLine)); //this returns only one action at a time
				}
				else if (newLine.contains("RESUME")){
					//resume statement
					header.setResumeHeaders(processResume(newLine));
				}
				else if (newLine.contains("AFTER")){
					//after statement
					header.setAfterHeaders(processAfter(newLine));
				}
				else if (newLine.contains("END")){
					//end of the header
					
					String headerName;
					headerScanner.useDelimiter("}");
					headerName = headerScanner.next();//getting the header name;
					headerName = headerName.replace("{", "");
					headerName = headerName.replace("END", "");
					headerName = headerName.trim();
					headerProcessCompleted = true;
				}
				else if (newLine.contains("BEGIN")){					
					//start of the header
					if (beginHeaderProcessed){//this is a nested header
						(header.getHeaders()).add(processHeader(scanner, newLine, header));
					}
					else{
						String headerName;
						headerScanner.useDelimiter("}");
						headerName = headerScanner.next();//getting the header name;
						headerName = headerName.replace("{", "");
						headerName = headerName.replace("BEGIN", "");
						headerName = headerName.trim();
						header.setName(headerName);
						beginHeaderProcessed = true;
					}
					
				}
				//use an if else chain to figure out what type of line it is
			} catch (StringIndexOutOfBoundsException e) {
				//do nothing
			}
			
			if (!headerProcessCompleted){
			newLine = scanner.next();
			}
			
		}
		
		if (header.getActions().size() == 0 && header.getHeaders().size() ==0){
			Action fakeAction = new Action();
			fakeAction.setFake(true);
			((ArrayList)(header.getActions())).add(fakeAction);
		}
		
		return header;
	}//end of processHeader function
	
	private String processIf(String anIf) {
		
		String ifCondition;
		Scanner ifScanner = new Scanner(anIf);
		ifScanner.useDelimiter("IF");
		ifCondition = ifScanner.next();
		ifCondition = ifCondition.trim();
		
		return ifCondition;
		
	}//end of processIf function


	private ArrayList processAfter(String anAfter) {
		ArrayList after = new ArrayList();
		Scanner afterScanner = new Scanner(anAfter);
		afterScanner.useDelimiter("}");
		
		while (afterScanner.hasNext()){
			String afterHeader = afterScanner.next();
			afterHeader = afterHeader.replace("AFTER"," ");
			afterHeader = afterHeader.replace("{"," ");
			afterHeader = afterHeader.trim();
			if (afterHeader.length() != 0){
				after.add(afterHeader);
			}
		}
		
		return after;
	}//end of processAfter function

	private ArrayList processResume(String aResume) {
		ArrayList resume = new ArrayList();
		Scanner resumeScanner = new Scanner(aResume);
		resumeScanner.useDelimiter("}");
	
		while (resumeScanner.hasNext()){
			String resumeHeader = resumeScanner.next();
			resumeHeader = resumeHeader.replace("RESUME"," ");
			resumeHeader = resumeHeader.replace("{"," ");
			resumeHeader = resumeHeader.trim();
			if (resumeHeader.length() != 0){
				resume.add(resumeHeader);
			}
			
		}
		
		return resume;
	}//end of processResume function
	

	

	public void createDecisionNodesForAlternativeFlows() throws NullPointerException{
		
		int i;	
		for (i = 0; i < useCases.size(); i++){
			
			try{
				UseCase currentUseCase = ((UseCase)useCases.get(i));
				AlternativeFlowSection AFSection = ((AlternativeFlowSection)currentUseCase.getAlternativeFlowSection()); 
				ArrayList AFs = ((ArrayList)AFSection.getAlternativeFlows());
				
				
				createEdgesToDecisionNodes(AFs, currentUseCase);
				
				int j;
				for ( j = 0 ; j < AFs.size() ; j++){
					AlternativeFlow alternativeFlow = ((AlternativeFlow)AFs.get(j));
					Header AFHeader = alternativeFlow.getTopLevelHeader();
					createActivityEdgesForTLHeader(AFHeader, currentUseCase);
				}
			} catch (NullPointerException e){
				//theres no alternative flows for this use case
			}
			
			
		}
	}
	
	public void createDecisionNodesForExtensionPoints() throws NullPointerException{
		
		int i;	
		for (i = 0; i < useCases.size(); i++){
			
			try {
				UseCase currentUseCase = ((UseCase)useCases.get(i));
				ExtensionPointSection EPSection = ((ExtensionPointSection)currentUseCase.getExtensionPointsSection()) ;
				ArrayList EPs = ((ArrayList)EPSection.getExtensionPoints());
				
				int j;
				
				for ( j = 0 ; j < EPs.size() ; j++) {
					ExtensionPoint currentEP = ((ExtensionPoint)EPs.get(j));
					createActivityEdgesToEPActivityDiagram(currentEP, currentUseCase, currentEP.getExtensionPointType());
				}
				
				for ( j = 0 ; j < EPs.size() ; j++){
					ExtensionPoint currentEP = ((ExtensionPoint)EPs.get(j));
					if(currentEP.getExtensionPointType().equalsIgnoreCase(currentEP.TYPE_privatePoint) ||
							currentEP.getExtensionPointType().equalsIgnoreCase(currentEP.TYPE_publicBehavior)){
						AlternativeFlow alternativeFlow = ((AlternativeFlow)currentEP.getAlternativeFlow());
						Header EPHeader = alternativeFlow.getTopLevelHeader();
						createActivityEdgesForTLHeader(EPHeader, currentUseCase);
					}
					
				}
				
				ArrayList activityDiagrams = currentUseCase.getActivityDiagrams();
				
				int k;
				for (k = 0 ; k < activityDiagrams.size(); k++){
					ActivityDiagram EPDiagram = ((ActivityDiagram)activityDiagrams.get(k));
					if (((EPDiagram.getType().equalsIgnoreCase(EPDiagram.TYPE_PRIVATE))) ||
							(EPDiagram.getType().equalsIgnoreCase(EPDiagram.TYPE_BEHAVIOR))){
						
						ControlNode startNode = new ControlNode ();
						startNode.setType(startNode.TYPE_initial);
						ControlNode finalNode = new ControlNode ();
						finalNode.setType(finalNode.TYPE_final);
						ControlNode joinFinalNode = new ControlNode();
						joinFinalNode.setType(joinFinalNode.TYPE_join);
						ActivityEdge activityEdge = new ActivityEdge();
						activityEdge.setSourceControlNode(joinFinalNode);
						activityEdge.setSourceType(activityEdge.TYPE_controlNode);
						activityEdge.setDestinationControlNode(finalNode);
						activityEdge.setDestinationType(activityEdge.TYPE_controlNode);
						activityEdges.add(activityEdge);
							
						createStartAndFinalNodes(EPDiagram , currentUseCase, startNode, joinFinalNode, finalNode);
					}
					
				}
			} catch (NullPointerException e) {
				//this use case does not have any extension points
			}
			
		}
	} 


	
	private void createActivityEdgesToEPActivityDiagram(ExtensionPoint currentEP, UseCase currentUseCase, String EP_TYPE) {
		
		//first off, its time to grab the right activity diagram
		
		ArrayList activityDiagrams = currentUseCase.getActivityDiagrams();
		
		int i;
		for (i = 0 ; i < activityDiagrams.size(); i++){
			ActivityDiagram currentActivityDiagram = (ActivityDiagram)activityDiagrams.get(i);
			
			if (EP_TYPE.equalsIgnoreCase(currentEP.TYPE_privatePoint) 
					&& currentActivityDiagram.getType().equalsIgnoreCase(currentActivityDiagram.TYPE_PRIVATE))	{
				
				AlternativeFlow currentAF = currentEP.getAlternativeFlow();
				String ifStatement = currentAF.getIfStatement();
				ArrayList atsList = currentAF.getAts();
				
				int k;
				for (k = 0; k < atsList.size(); k++){
					
					At ats = ((At)atsList.get(k));
					
					String atHeaderString =  ats.getAtHeader();
					Header realHeader = getHeaderFromString(atHeaderString, currentUseCase);
					
					
					if (ats.getActions().size() > 0){//we got an explicit list of actions
						int m;
						for (m = 0 ; m < ats.getActions().size(); m++){
							Action atAction = ((Action)ats.getActions().get(m));
							
							int n;
							for (n = 0 ; n < realHeader.getActions().size(); n ++){
								Action realAction = ((Action)realHeader.getActions().get(n));
								
								if (equalActions (realAction,atAction)){
									
									if (!realAction.isDecisionNodeCreated()){
										realAction.setDecisionNodeCreated(true);
										activityEdges.add(realAction.createDecisionNodeForThisAction());
									}
									ActivityEdge activityEdge = new ActivityEdge();
									activityEdge.setSourceControlNode(realAction.getDecisionNode());
									activityEdge.setSourceType(activityEdge.TYPE_controlNode);
									activityEdge.setDestinationActivityDiagram(currentActivityDiagram);
									activityEdge.setDestinationType(activityEdge.TYPE_activityDiagram);
									activityEdge.setIfStatement(ifStatement);
									activityEdges.add(activityEdge);

								}
							}
							
						}
						
					}
					else{
						int n;
						for (n = 0 ; n < realHeader.getActions().size(); n ++){
							Action realAction = ((Action)realHeader.getActions().get(n));

							if (!realAction.isDecisionNodeCreated()){
								activityEdges.add(realAction.createDecisionNodeForThisAction());
							}
							ActivityEdge activityEdge = new ActivityEdge();
							activityEdge.setSourceControlNode(realAction.getDecisionNode());
							activityEdge.setSourceType(activityEdge.TYPE_controlNode);
							activityEdge.setDestinationActivityDiagram(currentActivityDiagram);
							activityEdge.setDestinationType(activityEdge.TYPE_activityDiagram);
							activityEdge.setIfStatement(ifStatement);
							activityEdges.add(activityEdge);
							
						}
					}
					
				}	
				
				String continueHeaderName = currentEP.getContinueHeaderName();
				Header continueHeader = getHeaderFromString (continueHeaderName, currentUseCase);
				Action firstAction = getFirstActionInTLHeader(continueHeader);
				
				ActivityEdge continueActivityEdge = new ActivityEdge();
				continueActivityEdge.setSourceActivityDiagram(currentActivityDiagram);
				continueActivityEdge.setSourceType(continueActivityEdge.TYPE_activityDiagram);
				continueActivityEdge.setDestinationAction(firstAction);
				continueActivityEdge.setDestinationType(continueActivityEdge.TYPE_action);
				activityEdges.add(continueActivityEdge);

			}
			
			else if (EP_TYPE.equalsIgnoreCase(currentEP.TYPE_publicBehavior) 
					&& currentActivityDiagram.getType().equalsIgnoreCase(currentActivityDiagram.TYPE_BEHAVIOR))	{
				
				String baseUseCaseName = currentEP.getBaseUC();
				String baseUCExtensionPointName = currentEP.getExtensionPointName();
				
				//create an extend relationship between the base use case and the extending use case
				UseCase baseUseCase = getUseCaseFromString (baseUseCaseName);
				String extendingUseCaseName = currentActivityDiagram.getOwningUseCaseName();
				UseCase extendingUseCase = getUseCaseFromString (extendingUseCaseName);
				
				System.out.println("extending use case: " + extendingUseCaseName + " extended use case: " + baseUseCaseName
						+ " extending at: " + baseUCExtensionPointName); 
				
				Relationship relationship = new Relationship();
				relationship.setType(relationship.TYPE_EXTEND);
				relationship.setClientUseCase(extendingUseCase);
				relationship.setSupplierUseCase(baseUseCase);
				
				
				
			
				//time to get the activity diagram the represents the indicated public extension point declaration
				
				ArrayList baseUCActivityDiagrams = baseUseCase.getActivityDiagrams();
				
				
				int j;
				for (j = 0 ; j < baseUCActivityDiagrams.size(); j++){
					
					ActivityDiagram currentBaseActivityDiagram = ((ActivityDiagram) baseUCActivityDiagrams.get(j));
					
					if ((currentBaseActivityDiagram.getType().equalsIgnoreCase(currentBaseActivityDiagram.TYPE_PUBLIC) 
							&& currentBaseActivityDiagram.getName().equalsIgnoreCase(baseUCExtensionPointName))){
						//we got the public extension point activity diagram that we need
						
						AlternativeFlow currentAF = currentEP.getAlternativeFlow();
						String ifStatement = currentAF.getIfStatement();
						ArrayList atsList = currentAF.getAts();
						
						relationship.setCondition(ifStatement);
						relationship.setExtensionLocation(baseUCExtensionPointName);
						
						relationships.add(relationship);
						
						int k;
						for (k = 0; k < atsList.size(); k++){
							
							At ats = ((At)atsList.get(k));
							
							String atHeaderString =  ats.getAtHeader();
							Header realHeader = getHeaderFromString(atHeaderString, baseUseCase);
							
							
							if (ats.getActions().size() > 0){//we got an explicit list of actions
								int m;
								for (m = 0 ; m < ats.getActions().size(); m++){
									Action atAction = ((Action)ats.getActions().get(m));
									
									int n;
									for (n = 0 ; n < realHeader.getActions().size(); n ++){
										Action realAction = ((Action)realHeader.getActions().get(n));
										
										if (equalActions (realAction,atAction)){
											
											if (!realAction.isDecisionNodeCreated()){
												realAction.setDecisionNodeCreated(true);
												activityEdges.add(realAction.createDecisionNodeForThisAction());
											}
											ActivityEdge activityEdge = new ActivityEdge();
											activityEdge.setSourceControlNode(realAction.getDecisionNode());
											activityEdge.setSourceType(activityEdge.TYPE_controlNode);
											activityEdge.setDestinationActivityDiagram(currentBaseActivityDiagram);
											activityEdge.setDestinationType(activityEdge.TYPE_activityDiagram);
											activityEdge.setIfStatement(ifStatement);
											activityEdges.add(activityEdge);

										}
									}
									
								}
								
							}
							else{
								int n;
								for (n = 0 ; n < realHeader.getActions().size(); n ++){
									Action realAction = ((Action)realHeader.getActions().get(n));

									if (!realAction.isDecisionNodeCreated()){
										activityEdges.add(realAction.createDecisionNodeForThisAction());
									}
									ActivityEdge activityEdge = new ActivityEdge();
									activityEdge.setSourceControlNode(realAction.getDecisionNode());
									activityEdge.setSourceType(activityEdge.TYPE_controlNode);
									activityEdge.setDestinationActivityDiagram(currentBaseActivityDiagram);
									activityEdge.setDestinationType(activityEdge.TYPE_activityDiagram);
									activityEdge.setIfStatement(ifStatement);
									activityEdges.add(activityEdge);
									
								}
							}
							
						}
						
						String continueHeaderName = currentEP.getContinueHeaderName();
						Header continueHeader = getHeaderFromString (continueHeaderName, baseUseCase);
						Action firstAction = getFirstActionInTLHeader(continueHeader);
						
						ActivityEdge continueActivityEdge = new ActivityEdge();
						continueActivityEdge.setSourceActivityDiagram(currentBaseActivityDiagram);
						continueActivityEdge.setSourceType(continueActivityEdge.TYPE_activityDiagram);
						continueActivityEdge.setDestinationAction(firstAction);
						continueActivityEdge.setDestinationType(continueActivityEdge.TYPE_action);
						activityEdges.add(continueActivityEdge);
						
					}
					
					
				}
				
				
				
			}
			
			
			
		}
		
	}

	private UseCase getUseCaseFromString(String baseUseCaseName) {
		
		int i;
		for (i = 0 ; i < useCases.size(); i++){
			
			UseCase currentUseCase = ((UseCase)useCases.get(i));
			
			if ((((NameSection)currentUseCase.getNameSection()).getName()).equalsIgnoreCase(baseUseCaseName)){
				return currentUseCase;
			}
			
		}
		
		return null;
		
	}

	private void createEdgesToDecisionNodes(ArrayList AFs, UseCase currentUseCase) {
		
		int j;
		for ( j = 0 ; j < AFs.size() ; j++) {
			AlternativeFlow currentAF = ((AlternativeFlow)AFs.get(j));
			String ifStatement = currentAF.getIfStatement();
			ArrayList atsList = currentAF.getAts();
			
			int k;
			for (k = 0; k < atsList.size(); k++){
				At ats = ((At)atsList.get(k));
				
				String atHeaderString =  ats.getAtHeader();
				Header realHeader = getHeaderFromString(atHeaderString, currentUseCase);
				
				
				if (ats.getActions().size() > 0){//we got an explicit list of actions
					int m;
					for (m = 0 ; m < ats.getActions().size(); m++){
						Action atAction = ((Action)ats.getActions().get(m));
						
						int n;
						for (n = 0 ; n < realHeader.getActions().size(); n ++){
							Action realAction = ((Action)realHeader.getActions().get(n));
							
							if (equalActions (realAction,atAction)){
								
								if (!realAction.isDecisionNodeCreated()){
									realAction.setDecisionNodeCreated(true);
									activityEdges.add(realAction.createDecisionNodeForThisAction());
								}
								ActivityEdge activityEdge = new ActivityEdge();
								activityEdge.setSourceControlNode(realAction.getDecisionNode());
								activityEdge.setSourceType(activityEdge.TYPE_controlNode);
								activityEdge.setDestinationAction(getFirstActionInTLHeader(currentAF.getTopLevelHeader()));
								activityEdge.setDestinationType(activityEdge.TYPE_action);
								activityEdge.setIfStatement(ifStatement);
								activityEdges.add(activityEdge);

							}
						}
						
					}
					
				}
				else{
					int n;
					for (n = 0 ; n < realHeader.getActions().size(); n ++){
						Action realAction = ((Action)realHeader.getActions().get(n));

						if (!realAction.isDecisionNodeCreated()){
							activityEdges.add(realAction.createDecisionNodeForThisAction());
						}
						ActivityEdge activityEdge = new ActivityEdge();
						activityEdge.setSourceControlNode(realAction.getDecisionNode());
						activityEdge.setSourceType(activityEdge.TYPE_controlNode);
						activityEdge.setDestinationAction(getFirstActionInTLHeader(currentAF.getTopLevelHeader()));
						activityEdge.setDestinationType(activityEdge.TYPE_action);
						activityEdge.setIfStatement(ifStatement);
						activityEdges.add(activityEdge);
						
					}
				}
				
				
			}
			
			
		}
		
	}

	private boolean equalActions(Action realAction, Action atAction) {
		
		if (realAction.getActor().equalsIgnoreCase(atAction.getActor()) &&
				realAction.getDescription().equalsIgnoreCase(atAction.getDescription())){
			return true;
		}
		else {
			return false;
		}
		
	}

	private Action getFirstActionInTLHeader(Header TLHeader) {
		
		if (TLHeader.getActions().size() > 0 ){
			return ((Action)TLHeader.getActions().get(0)); //get the first action only
		}
		else if (TLHeader.getHeaders().size() > 0){
			
			int i;
			for (i = 0 ; i < TLHeader.getHeaders().size() ; i++){
				Header nextHeader = ((Header)TLHeader.getHeaders().get(i));
				try{
					return getFirstActionInTLHeader(nextHeader);
				} catch (NullPointerException e){
					
				}
			}
		}
		
		return null;
	}


	private Header getHeaderFromString(String header, UseCase currentUseCase) {
		
		//---first look into thebasic flow
		Header BFHeader = ((BasicFlowSection)currentUseCase.getBasicSection()).getTopLevelHeader();
		
		
		try{
			return getHeaderFromStringFromTLH(header, BFHeader);
		} catch (NullPointerException e){
			
		}
		
		
		int i;
		//---then look into Alternative flows
		SubflowSection SFSection = ((SubflowSection)currentUseCase.getSubflowSection());
		ArrayList SFs = ((ArrayList)SFSection.getSubflows());

		for(i = 0; i < SFs.size(); i++){
			Subflow currentSF = ((Subflow)SFs.get(i));
			Header SFHeader = currentSF.getTopLevelHeader();
			try{
				return getHeaderFromStringFromTLH(header, SFHeader);
			} catch (NullPointerException e){
				
			}
		}
		
		//---then look into extension points
		ExtensionPointSection EPSection = ((ExtensionPointSection)currentUseCase.getExtensionPointsSection());
		ArrayList EPs = ((ArrayList)EPSection.getExtensionPoints());

		for(i = 0; i < EPs.size(); i++){
			ExtensionPoint currentEP = ((ExtensionPoint)EPs.get(i));
			
			if (currentEP.getExtensionPointType().equalsIgnoreCase(currentEP.TYPE_privatePoint) ||
					currentEP.getExtensionPointType().equalsIgnoreCase(currentEP.TYPE_publicBehavior)){
				
				Header EPHeader = (Header)((AlternativeFlow)currentEP.getAlternativeFlow()).getTopLevelHeader();
				
				try{
					return getHeaderFromStringFromTLH(header, EPHeader);
				} catch (NullPointerException e){
					
				}
			}
		}
		
		//---then look into Alternative flows
		AlternativeFlowSection AFSection = ((AlternativeFlowSection)currentUseCase.getAlternativeFlowSection()); 
		ArrayList AFs = ((ArrayList)AFSection.getAlternativeFlows());

		for(i = 0; i < AFs.size(); i++){
			AlternativeFlow currentAF = ((AlternativeFlow)AFs.get(i));
			Header AFHeader = currentAF.getTopLevelHeader();
			try{
				return getHeaderFromStringFromTLH(header, AFHeader);
			} catch (NullPointerException e){
				
			}
		}
		
		return null;
	}

	private Header getHeaderFromStringFromTLH(String header, Header TLHeader) {
		
		if (TLHeader.getActions().size() > 0 ){
			if (TLHeader.getName().equalsIgnoreCase(header)){
				return TLHeader;
			}
		}
		else if (TLHeader.getHeaders().size() > 0){
			
			int i;
			for (i = 0 ; i < TLHeader.getHeaders().size() ; i++){
				Header nextHeader = ((Header)TLHeader.getHeaders().get(i));
				try{
					Header requestedHeader  = getHeaderFromStringFromTLH(header, nextHeader);
					requestedHeader.getName();//forcing the null pointer exception to be raised to be caught later
					return getHeaderFromStringFromTLH(header, nextHeader);
				} catch (NullPointerException e){
					
				}
			}
		}
		
		return null;
	}
	
	


	public void createActivityEdgesForBasicFlows() {
		try {
			int i;	
			for (i = 0; i < useCases.size(); i++){
				
				UseCase currentUseCase = ((UseCase)useCases.get(i));
				Header BFHeader = ((BasicFlowSection)currentUseCase.getBasicSection()).getTopLevelHeader();
				
				createActivityEdgesForTLHeader(BFHeader, currentUseCase);
				
				ArrayList activityDiagrams = currentUseCase.getActivityDiagrams();
				
				int j;
				for (j = 0 ; j < activityDiagrams.size(); j++){
					ActivityDiagram basicFlowDiagram = ((ActivityDiagram)activityDiagrams.get(j));
					if (basicFlowDiagram.getType().equalsIgnoreCase(basicFlowDiagram.TYPE_BASIC)){
				
						ControlNode startNode = new ControlNode ();
						startNode.setType(startNode.TYPE_initial);
						ControlNode finalNode = new ControlNode ();
						finalNode.setType(finalNode.TYPE_final);
						ControlNode joinFinalNode = new ControlNode();
						joinFinalNode.setType(joinFinalNode.TYPE_join);
						ActivityEdge activityEdge = new ActivityEdge();
						activityEdge.setSourceControlNode(joinFinalNode);
						activityEdge.setSourceType(activityEdge.TYPE_controlNode);
						activityEdge.setDestinationControlNode(finalNode);
						activityEdge.setDestinationType(activityEdge.TYPE_controlNode);
						activityEdges.add(activityEdge);
						
						
						createStartAndFinalNodes(basicFlowDiagram , currentUseCase, startNode, joinFinalNode, finalNode);
					}
					
				}
				
				

			}
		} catch (NullPointerException e) {
			//no basic flow in this use case
		}
		
	}
	


	public void createActivityEdgesForSubflows() {
		
		int i;	
		for (i = 0; i < useCases.size(); i++){
			try {
				UseCase currentUseCase = ((UseCase)useCases.get(i));
				SubflowSection SFSection = ((SubflowSection)currentUseCase.getSubflowSection());
				ArrayList subflowsList = ((ArrayList)SFSection.getSubflows());
				
				int j;
				for (j = 0 ; j < subflowsList.size() ; j++){
					
					Subflow currentSubflow = ((Subflow)subflowsList.get(j));
					Header SFHeader = currentSubflow.getTopLevelHeader();
					createActivityEdgesForTLHeader(SFHeader, currentUseCase);
					
				}
				
				ArrayList activityDiagrams = currentUseCase.getActivityDiagrams();
				
				int k;
				for (k = 0 ; k < activityDiagrams.size(); k++){
					ActivityDiagram subflowDiagram = ((ActivityDiagram)activityDiagrams.get(k));
					if (subflowDiagram.getType().equalsIgnoreCase(subflowDiagram.TYPE_SUBFLOW)){
						
						ControlNode startNode = new ControlNode ();
						startNode.setType(startNode.TYPE_initial);
						ControlNode finalNode = new ControlNode ();
						finalNode.setType(finalNode.TYPE_final);
						ControlNode joinFinalNode = new ControlNode();
						joinFinalNode.setType(joinFinalNode.TYPE_join);
						ActivityEdge activityEdge = new ActivityEdge();
						activityEdge.setSourceControlNode(joinFinalNode);
						activityEdge.setSourceType(activityEdge.TYPE_controlNode);
						activityEdge.setDestinationControlNode(finalNode);
						activityEdge.setDestinationType(activityEdge.TYPE_controlNode);
						activityEdges.add(activityEdge);
						
						
						createStartAndFinalNodes(subflowDiagram , currentUseCase, startNode, joinFinalNode, finalNode);
					}
					
				}
				
			}
			catch (NullPointerException e) {
				//no subflows in this use case
			}
		} 
		
	}
	
	private void createStartAndFinalNodes(ActivityDiagram activityDiagram, UseCase currentUseCase, ControlNode startNode, ControlNode joinNode, ControlNode finalNode) {
		
		ArrayList actions = ((ArrayList)activityDiagram.getActions());
		
		finalNode.setOwningActivityDiagram(activityDiagram);
		finalNode.setControlID(activityDiagram.getNodeID());
		activityDiagram.setNodeID(activityDiagram.getNodeID() + 1);
		
		startNode.setOwningActivityDiagram(activityDiagram);
		startNode.setControlID(activityDiagram.getNodeID());
		activityDiagram.setNodeID(activityDiagram.getNodeID()+ 1);
		
		joinNode.setOwningActivityDiagram(activityDiagram);
		joinNode.setControlID(activityDiagram.getNodeID());
		activityDiagram.setNodeID(activityDiagram.getNodeID()+ 1);
		
		int j;
		for ( j = 0 ; j < actions.size() ; j++){
			Action action = ((Action)actions.get(j));
			if (!isASourceForAnEdge(action)){
				ActivityEdge activityEdge = new ActivityEdge();
				activityEdge.setSourceAction(action);
				activityEdge.setSourceType(activityEdge.TYPE_action);
				activityEdge.setDestinationControlNode(joinNode);
				activityEdge.setDestinationType(activityEdge.TYPE_controlNode);
				activityEdges.add(activityEdge);
			}
			
			if (action.isGoesToFinalNode()){
				
				ActivityEdge activityEdge = new ActivityEdge();
				if (action.isForkNodeCreated()){
					activityEdge.setSourceControlNode(action.getForkNode());
					activityEdge.setSourceType(activityEdge.TYPE_controlNode);
					activityEdge.setDestinationControlNode(joinNode);
					activityEdge.setDestinationType(activityEdge.TYPE_controlNode);
					activityEdges.add(activityEdge);
				}
				else if (action.isDecisionNodeCreated()){
					activityEdge.setSourceControlNode(action.getDecisionNode());
					activityEdge.setSourceType(activityEdge.TYPE_controlNode);
					activityEdge.setDestinationControlNode(joinNode);
					activityEdge.setDestinationType(activityEdge.TYPE_controlNode);
					activityEdges.add(activityEdge);
				}
				else{
					activityEdge.setSourceAction(action);
					activityEdge.setSourceType(activityEdge.TYPE_action);
					activityEdge.setDestinationControlNode(joinNode);
					activityEdge.setDestinationType(activityEdge.TYPE_controlNode);
					activityEdges.add(activityEdge);
				}
			}
			
			if (!isADestinationForAnEdge(action) ){
				ActivityEdge activityEdge = new ActivityEdge();
				activityEdge.setDestinationAction(action);
				activityEdge.setDestinationType(activityEdge.TYPE_action);
				activityEdge.setSourceControlNode(startNode);
				activityEdge.setSourceType(activityEdge.TYPE_controlNode);
				activityEdges.add(activityEdge);
			}
			
			
		}
	}


	private boolean isADestinationForAnEdge(Action action) {
		
		int i;
		for ( i = 0 ; i < activityEdges.size() ; i++){
			ActivityEdge activityEdge = ((ActivityEdge)activityEdges.get(i));

			if (activityEdge.getDestinationType().equalsIgnoreCase(activityEdge.TYPE_action) &&
					activityEdge.getDestinationAction().equals(action)){
				return true;
			}
			
		}
		
		return false;
	}

	private boolean isASourceForAnEdge(Action action) {
		
		int i;
		for ( i = 0 ; i < activityEdges.size() ; i ++){
			ActivityEdge activityEdge = ((ActivityEdge)activityEdges.get(i));
			
			if (activityEdge.getSourceType().equalsIgnoreCase(activityEdge.TYPE_action) &&
					activityEdge.getSourceAction().equals(action)){
				return true;
			}
		}
		
		return false;
	}

	private void createActivityEdgesForTLHeader(Header header, UseCase currentUseCase) {


		if (header.getActions().size() > 0){//this header contains actions

			createActivityEdgesWithinHeader (header);//create edges between actions
			createActivityEdgesAtResume(header, currentUseCase);//create edges for the last action
			if (header.getAfterHeaders().size() != 0){
				createActivityEdgesAtAfter (header, currentUseCase);
			}

		}
		else if (header.getHeaders().size() > 0){//this header contains other headers
			int i;
			for (i = 0 ; i < header.getHeaders().size() ; i++){
				Header nextHeader = ((Header)header.getHeaders().get(i));
				try{
					createActivityEdgesForTLHeader(nextHeader, currentUseCase);
				} catch (NullPointerException e){
					
				}
			}
		}

	}

	private void createActivityEdgesAtAfter(Header header, UseCase currentUseCase) {
		
		ArrayList afterHeaders = (ArrayList)header.getAfterHeaders();
		
		if (afterHeaders.size() > 1){//there's a need to create a join node
			
			Action firstAction = (Action)header.getActions().get(0);//get the first action in the header
			ControlNode joinNode= new ControlNode ();
			joinNode.setOwnerAction(firstAction);
			joinNode.setOwningActivityDiagram(firstAction.getOwningActivityDiagram());
			joinNode.setControlID(firstAction.getOwningActivityDiagram().getNodeID());
			firstAction.getOwningActivityDiagram().setNodeID(firstAction.getOwningActivityDiagram().getNodeID() + 1 );
			ActivityEdge activityEdge = new ActivityEdge();
			
			joinNode.setType(joinNode.TYPE_join);
			activityEdge.setSourceControlNode(joinNode);
			activityEdge.setSourceType(activityEdge.TYPE_controlNode);
			activityEdge.setDestinationAction(firstAction);
			activityEdge.setDestinationType(activityEdge.TYPE_action);
			activityEdges.add(activityEdge);
			
			
			int i;
			for (i = 0; i < afterHeaders.size(); i++){
				String afterHeaderName = ((String)afterHeaders.get(i));
				Header afterHeader = getHeaderFromString (afterHeaderName, currentUseCase);
				ActivityEdge lastEdge = getLastEdge (afterHeader, firstAction);
				
				lastEdge.setDestinationAction(null);
				lastEdge.setDestinationControlNode(joinNode);
				lastEdge.setDestinationType(lastEdge.TYPE_controlNode);
			}
		}
		
		
		
	}

	private ActivityEdge getLastEdge(Header afterHeader, Action firstAction) {
		
		//getting the last action in the previous header
		Action lastAction = (Action)afterHeader.getActions().get(afterHeader.getActions().size()-1);
		int i;
		
		if (lastAction.isForkNodeCreated()){
			
			for (i = 0 ; i < activityEdges.size(); i++){
				
				ActivityEdge lastEdge = ((ActivityEdge)activityEdges.get(i));
				
				try {
					if (lastEdge.getSourceControlNode().equals(lastAction.getForkNode())
							&& lastEdge.getDestinationAction().equals(firstAction)){
						return lastEdge;
					}
				} catch (NullPointerException e) {
					
				}
				
			}
			
		}
		else if(lastAction.isDecisionNodeCreated()){

			
			for (i = 0 ; i < activityEdges.size(); i++){
				
				ActivityEdge lastEdge = ((ActivityEdge)activityEdges.get(i));
				
				try {
					if (lastEdge.getSourceControlNode().equals(lastAction.getDecisionNode())
							&& lastEdge.getDestinationAction().equals(firstAction)){
						return lastEdge;
					}
				} catch (NullPointerException e) {
					
				}
				
			}
		}
		else{
			//must be connected directly to the action
			for (i = 0 ; i < activityEdges.size(); i++){
				
				ActivityEdge lastEdge = ((ActivityEdge)activityEdges.get(i));
				
				try {
					if (lastEdge.getSourceAction().equals(lastAction)
							&& lastEdge.getDestinationAction().equals(firstAction)){
						return lastEdge;
					}
				} catch (NullPointerException e) {
					
				}
				
			}
		}
		
		return null;
	}

	private void createActivityEdgesAtResume(Header header, UseCase currentUseCase) {
		ArrayList actions = header.getActions();
		Action lastAction = (Action)actions.get(actions.size()-1);
		
		if (HeaderHasResume(header)){//this header has a resume statement
			ArrayList resumeHeaders = header.getResumeHeaders();
			
			if (resumeHeaders.size() == 1 ){//only one resume header, no need for a fork node
				String resumeHeaderName = ((String)resumeHeaders.get(0));
				
				if (!resumeHeaderName.equalsIgnoreCase("END")){
					Header resumeHeader = getHeaderFromString(resumeHeaderName, currentUseCase);
					
					
					if(lastAction.isDecisionNodeCreated()){
						ActivityEdge activityEdge = new ActivityEdge ();
						activityEdge.setSourceControlNode(lastAction.getDecisionNode());
						activityEdge.setSourceType(activityEdge.TYPE_controlNode);
						activityEdge.setDestinationAction(getFirstActionInTLHeader(resumeHeader));
						activityEdge.setDestinationType(activityEdge.TYPE_action);
						activityEdges.add(activityEdge);
					}
					else{
						ActivityEdge activityEdge = new ActivityEdge ();
						activityEdge.setSourceAction(lastAction);
						activityEdge.setSourceType(activityEdge.TYPE_action);
						activityEdge.setDestinationAction(getFirstActionInTLHeader(resumeHeader));
						activityEdge.setDestinationType(activityEdge.TYPE_action);
						activityEdges.add(activityEdge);
					}
				}
				else{
					lastAction.setGoesToFinalNode(true);
				}
				
			}
			else if (resumeHeaders.size() > 1){//more than one resume header, requires a fork node
				
				int j;
				for (j = 0 ; j < resumeHeaders.size(); j++) {
					String resumeHeaderName = ((String)resumeHeaders.get(j));
					
					if (!resumeHeaderName.equalsIgnoreCase("END")){
						Header resumeHeader = getHeaderFromString(resumeHeaderName, currentUseCase);
						
						if(!lastAction.isForkNodeCreated()){
							activityEdges.add(lastAction.createForkNodeForThisAction());
							
						}
						
						//connect the fork node to the next action in the next header
						ActivityEdge activityEdge = new ActivityEdge ();
						activityEdge.setSourceControlNode(lastAction.getForkNode());
						activityEdge.setSourceType(activityEdge.TYPE_controlNode);
						activityEdge.setDestinationAction(getFirstActionInTLHeader(resumeHeader));
						activityEdge.setDestinationType(activityEdge.TYPE_action);
						activityEdges.add(activityEdge);
					}
					else{
						lastAction.setGoesToFinalNode(true);
					}
				}

			}
		}
		else{//this header does not have a resume statement
			//look for the first action of the next header
			try {
				Header nextHeader =  getNextHeader(header);
				Action nextAction = getFirstActionInTLHeader(nextHeader);
				
				ActivityEdge activityEdge = new ActivityEdge ();
				activityEdge.setSourceAction(lastAction);
				activityEdge.setSourceType(activityEdge.TYPE_action);
				activityEdge.setDestinationAction(nextAction);
				activityEdge.setDestinationType(activityEdge.TYPE_action);
				activityEdges.add(activityEdge);
				
			} catch (NullPointerException e) {//theres no next header
				//we'll do nothing here, at the end we will make a function that will
				//look for nodes that don't have outgoing edges and will connect those using edges
				//to a single final node.
			}
		}
	}

	private Header getNextHeader(Header header) {
		
		Header TLHeader = header.getParentHeader();
		ArrayList nextHeaders = TLHeader.getHeaders();
		
		int i;
		for (i = 0 ; i < nextHeaders.size(); i++){
			Header currentHeader = ((Header)nextHeaders.get(i));
			Header lastHeader = ((Header)nextHeaders.get(nextHeaders.size()-1));
			if (equalHeaders(currentHeader, header)){
				//check it is the last header?
			
				if (equalHeaders(currentHeader, lastHeader) ){
					
					return getNextHeader (TLHeader);
				}
				else{//its not the last header, just return the next header
					
					Header nextHeader = ((Header)nextHeaders.get(i+1));
					return nextHeader;
				}
			}
			
		}
		
		return null;
	}

	private boolean equalHeaders(Header currentHeader, Header header) {
		
		if (currentHeader.getName().equalsIgnoreCase(header.getName())){
			return true;
		}
		else{
			return false;
		}
		
	}

	private boolean HeaderHasResume(Header header) {
		
		if (header.getResumeHeaders().size()>0){
			return true;
		}
		else{
			return false;
		}
		
	}


	private void createActivityEdgesWithinHeader(Header header) {
		
		ArrayList actions = header.getActions();
		int i;
		
		for ( i = 0 ; i < actions.size() - 1 ; i++ ){
			Action currentAction = ((Action)actions.get(i));
			Action nextAction = ((Action)actions.get(i+1));
			
			if(currentAction.isDecisionNodeCreated()){
				ActivityEdge activityEdge = new ActivityEdge ();
				activityEdge.setSourceControlNode(currentAction.getDecisionNode());
				activityEdge.setSourceType(activityEdge.TYPE_controlNode);
				activityEdge.setDestinationAction(nextAction);
				activityEdge.setDestinationType(activityEdge.TYPE_action);
				activityEdges.add(activityEdge);
			}
			else{
				ActivityEdge activityEdge = new ActivityEdge ();
				activityEdge.setSourceAction(currentAction);
				activityEdge.setSourceType(activityEdge.TYPE_action);
				activityEdge.setDestinationAction(nextAction);
				activityEdge.setDestinationType(activityEdge.TYPE_action);
				activityEdges.add(activityEdge);
			}
			
		}
	}

	public void createActivityDiagrams() {
		
		
		int i;	
		for (i = 0; i < useCases.size(); i++){
			UseCase currentUseCase = ((UseCase)useCases.get(i));
			
			try {
				
				Header BFHeader = ((BasicFlowSection)currentUseCase.getBasicSection()).getTopLevelHeader();
				ActivityDiagram basicFlowDiagram = new ActivityDiagram ();
				ArrayList BFActions = new ArrayList ();
				
				getAllActionsInHeader(BFHeader, BFActions, basicFlowDiagram);
				basicFlowDiagram.setActions(BFActions);
				basicFlowDiagram.setName("Basic Flow");
				basicFlowDiagram.setType(basicFlowDiagram.TYPE_BASIC);
				basicFlowDiagram.setOwningUseCaseName(((NameSection)currentUseCase.getNameSection()).getName());
				((ArrayList)currentUseCase.getActivityDiagrams()).add(basicFlowDiagram);
			} catch (NullPointerException e){
				
			}
			
			try {
				
				SubflowSection SFSection = ((SubflowSection)currentUseCase.getSubflowSection());
				ArrayList subflowsList = ((ArrayList)SFSection.getSubflows());
				
				
				int j;
				for (j = 0 ; j < subflowsList.size() ; j++){
					
					Subflow currentSubflow = ((Subflow)subflowsList.get(j));
					Header SFHeader = currentSubflow.getTopLevelHeader();
					ActivityDiagram subflowDiagram = new ActivityDiagram ();
					subflowDiagram.setName(currentSubflow.getName());
					subflowDiagram.setType(subflowDiagram.TYPE_SUBFLOW);
					ArrayList SFActions = new ArrayList ();
					
					getAllActionsInHeader(SFHeader, SFActions, subflowDiagram);
					subflowDiagram.setActions(SFActions);
					subflowDiagram.setOwningUseCaseName(((NameSection)currentUseCase.getNameSection()).getName());
					((ArrayList)currentUseCase.getActivityDiagrams()).add(subflowDiagram);
				}
			}
			catch (NullPointerException e){
				
			}
				
				
			try {
				
				AlternativeFlowSection AFSection = ((AlternativeFlowSection)currentUseCase.getAlternativeFlowSection()); 
				ArrayList AFs = ((ArrayList)AFSection.getAlternativeFlows());
				
				int j;
				for (j = 0 ; j < AFs.size(); j++ ){
					AlternativeFlow currentAF = ((AlternativeFlow)AFs.get(j));
					String flowName = currentAF.getFlow();
					
					ArrayList activityDiagrams = ((ArrayList)currentUseCase.getActivityDiagrams());
					int k;
					for ( k = 0 ; k < activityDiagrams.size() ; k++){
						ActivityDiagram currentActivityDiagram = ((ActivityDiagram)activityDiagrams.get(k));
						
						if (flowName.equals(currentActivityDiagram.getName())){
							ArrayList AFActions = new ArrayList ();
							Header AFHeader = currentAF.getTopLevelHeader();
							
							getAllActionsInHeader(AFHeader, AFActions, currentActivityDiagram);
							
							int m;
							for (m = 0 ; m < AFActions.size(); m++){
								((ArrayList)currentActivityDiagram.getActions()).add((Action)AFActions.get(m));
							}
						}
					}
				}
			} catch (NullPointerException e){
				
			}
				
			try{
				ExtensionPointSection EPSection = ((ExtensionPointSection)currentUseCase.getExtensionPointsSection()) ;
				ArrayList EPs = ((ArrayList)EPSection.getExtensionPoints());
				
				int j;
				for (j = 0 ; j < EPs.size(); j++ ){
					
					ExtensionPoint currentExtensionPoint = ((ExtensionPoint)EPs.get(j));
					
					if (currentExtensionPoint.getExtensionPointType().equalsIgnoreCase(currentExtensionPoint.TYPE_privatePoint) ||
							currentExtensionPoint.getExtensionPointType().equalsIgnoreCase(currentExtensionPoint.TYPE_publicBehavior)){
						
						Header extensionAFHeader = ((AlternativeFlow)currentExtensionPoint.getAlternativeFlow()).getTopLevelHeader();
						ActivityDiagram extensionPointDiagram = new ActivityDiagram ();
						extensionPointDiagram.setName(extensionAFHeader.getName());
						if (currentExtensionPoint.getExtensionPointType().equalsIgnoreCase(currentExtensionPoint.TYPE_privatePoint)){
							extensionPointDiagram.setType(extensionPointDiagram.TYPE_PRIVATE);
						}
						else{
							extensionPointDiagram.setType(extensionPointDiagram.TYPE_BEHAVIOR);
						}
						extensionPointDiagram.setOwningUseCaseName(((NameSection)currentUseCase.getNameSection()).getName());
						ArrayList EPActions = new ArrayList ();
						getAllActionsInHeader(extensionAFHeader, EPActions, extensionPointDiagram);
						extensionPointDiagram.setActions(EPActions);
						((ArrayList)currentUseCase.getActivityDiagrams()).add(extensionPointDiagram);
					}
					else if (currentExtensionPoint.getExtensionPointType().equalsIgnoreCase(currentExtensionPoint.TYPE_publicDeclaration)){
						
						ActivityDiagram extensionPointDiagram = new ActivityDiagram ();
						extensionPointDiagram.setName(currentExtensionPoint.getExtensionPointName());
						extensionPointDiagram.setType(extensionPointDiagram.TYPE_PUBLIC);
						extensionPointDiagram.setOwningUseCaseName(((NameSection)currentUseCase.getNameSection()).getName());
						((ArrayList)currentUseCase.getActivityDiagrams()).add(extensionPointDiagram);
					}
					
					
				}
				
			}
			catch (NullPointerException e){
				
				
			}
		}
		
	}//end of create activity diagrams function

	private void getAllActionsInHeader(Header header, ArrayList actions, ActivityDiagram owningActivityDiagram) {
		
		
		if (header.getActions().size()> 0){
			
			int j;
			for ( j = 0 ; j < header.getActions().size() ; j++){
				((Action)header.getActions().get(j)).setOwningActivityDiagram(owningActivityDiagram);
				((Action)header.getActions().get(j)).setActionID(owningActivityDiagram.getNodeID());
				owningActivityDiagram.setNodeID(owningActivityDiagram.getNodeID() + 1);
				actions.add((Action)header.getActions().get(j));
			}
			return;
		}
		else if (header.getHeaders().size()>0){
			
			int i;
			for (i = 0 ; i < header.getHeaders().size(); i++){
				getAllActionsInHeader ((Header)header.getHeaders().get(i), actions, owningActivityDiagram);
			}
		}
		
		
		
	}

	public void removeFakeNodes() {
		
		int i;
		for ( i = 0 ; i < activityEdges.size(); i++){
			
			ActivityEdge incomingActivityEdge = ((ActivityEdge)activityEdges.get(i));
			
			if (incomingActivityEdge.getDestinationType().equalsIgnoreCase(incomingActivityEdge.TYPE_action) &&
					incomingActivityEdge.getDestinationAction().isFake()){

				Action action = incomingActivityEdge.getDestinationAction();
				
				int j;
				for ( j = 0 ; j < activityEdges.size(); j++){
					
					ActivityEdge outgoingActivityEdge = ((ActivityEdge)activityEdges.get(j));
					
					if (outgoingActivityEdge.getSourceType().equalsIgnoreCase(outgoingActivityEdge.TYPE_action) &&
							outgoingActivityEdge.getSourceAction().equals(action)){
						
						int k;
						for (k = 0 ; k < activityEdges.size(); k++){
							
							ActivityEdge incomingEdge = ((ActivityEdge)activityEdges.get(k));
							
							if (incomingEdge.getDestinationType().equalsIgnoreCase(incomingEdge.TYPE_action) &&
									incomingEdge.getDestinationAction().equals(action)){
								
								if (outgoingActivityEdge.getDestinationType().equalsIgnoreCase(outgoingActivityEdge.TYPE_action)){
									incomingEdge.setDestinationAction(outgoingActivityEdge.getDestinationAction());
								}
								else if (outgoingActivityEdge.getDestinationType().equalsIgnoreCase(outgoingActivityEdge.TYPE_controlNode)){
									incomingEdge.setDestinationAction(null);
									incomingEdge.setDestinationControlNode(outgoingActivityEdge.getDestinationControlNode());
									incomingEdge.setDestinationType(incomingEdge.TYPE_controlNode);
								}
								else if (outgoingActivityEdge.getDestinationType().equalsIgnoreCase(outgoingActivityEdge.TYPE_activityDiagram)){
									incomingEdge.setDestinationAction(null);
									incomingEdge.setSourceActivityDiagram(outgoingActivityEdge.getDestinationActivityDiagram());
									incomingEdge.setDestinationType(incomingEdge.TYPE_activityDiagram);
								}
								
							}
							
						}
						
						activityEdges.remove(j);
						i = -1;//restart the main loop
						break;
					}
				
				}
			}
			
		}
	}

	public void removeUnrequiredJoinNodes() {

		int i;
		for ( i = 0 ; i < activityEdges.size(); i++){
			
			ActivityEdge incomingActivityEdge = ((ActivityEdge)activityEdges.get(i));
			if (incomingActivityEdge.getDestinationType().equalsIgnoreCase(incomingActivityEdge.TYPE_controlNode)){
				
				ControlNode joinNode = incomingActivityEdge.getDestinationControlNode();
				
				if (joinNode.getType().equalsIgnoreCase(joinNode.TYPE_join)){
					
					int incomingEdgesCounter = 0;
					int j;
					for ( j = 0 ; j < activityEdges.size(); j++){
						ActivityEdge activityEdge = ((ActivityEdge)activityEdges.get(j));
							
						if (activityEdge.getDestinationType().equalsIgnoreCase(activityEdge.TYPE_controlNode) &&
								activityEdge.getDestinationControlNode().equals(joinNode)){
							incomingEdgesCounter++;
						}

						
					}
					
					
					
					if (incomingEdgesCounter == 1){ //we need to remove the join Node
						
						int k;
						for ( k = 0 ; k < activityEdges.size(); k++){
							ActivityEdge outgoingActivityEdge = ((ActivityEdge)activityEdges.get(k));
								
							if (outgoingActivityEdge.getSourceType().equalsIgnoreCase(outgoingActivityEdge.TYPE_controlNode) &&
									outgoingActivityEdge.getSourceControlNode().equals(joinNode)){
								
								if (outgoingActivityEdge.getDestinationType().equalsIgnoreCase(outgoingActivityEdge.TYPE_action)){
									incomingActivityEdge.setDestinationActivityDiagram(null);
									incomingActivityEdge.setDestinationControlNode(null);
									incomingActivityEdge.setDestinationAction(outgoingActivityEdge.getDestinationAction());
									incomingActivityEdge.setDestinationType(incomingActivityEdge.TYPE_action);
								}
								else if (outgoingActivityEdge.getDestinationType().equalsIgnoreCase(outgoingActivityEdge.TYPE_controlNode)){
									incomingActivityEdge.setDestinationAction(null);
									incomingActivityEdge.setDestinationActivityDiagram(null);
									incomingActivityEdge.setDestinationControlNode(outgoingActivityEdge.getDestinationControlNode());
									incomingActivityEdge.setDestinationType(incomingActivityEdge.TYPE_controlNode);
								}
								else if (outgoingActivityEdge.getDestinationType().equalsIgnoreCase(outgoingActivityEdge.TYPE_activityDiagram)){
									incomingActivityEdge.setDestinationAction(null);
									incomingActivityEdge.setDestinationControlNode(null);
									incomingActivityEdge.setSourceActivityDiagram(outgoingActivityEdge.getDestinationActivityDiagram());
									incomingActivityEdge.setDestinationType(incomingActivityEdge.TYPE_activityDiagram);
								}
								
								activityEdges.remove(k);
								i = -1;
								break;
							}

							
						}
						
					}
					
				}
			}
			
			
		}
	}
	
	
	//----setters and getters-----
	public ArrayList getActivityEdges() {
		return activityEdges;
	}

	public void setActivityEdges(ArrayList activityEdges) {
		this.activityEdges = activityEdges;
	}

	public ArrayList getActors() {
		return actors;
	}

	public void setActors(ArrayList actors) {
		this.actors = actors;
	}


	public ArrayList getUseCases() {
		return useCases;
	}

	public void setUseCases(ArrayList useCases) {
		this.useCases = useCases;
	}
	
	public ArrayList getRelationships() {
		return relationships;
	}

	public void setRelationships(ArrayList relationships) {
		this.relationships = relationships;
	}
	//--end of setters and getters----

	public void createPerformRelationships() {
		
		int i;	
		for (i = 0; i < useCases.size(); i++){
			try {
				UseCase currentUseCase = ((UseCase)useCases.get(i));
				ArrayList activityDiagrams = currentUseCase.getActivityDiagrams();
				
				int j;
				for (j = 0 ; j < activityDiagrams.size(); j++){
					
					ActivityDiagram currentActivityDiagram = (ActivityDiagram)activityDiagrams.get(j);
					ArrayList actions = ((ArrayList)currentActivityDiagram.getActions());

					int k;
					for (k = 0 ; k < actions.size(); k++){
						
						Action currentAction = ((Action)actions.get(k)) ;
						
						if (currentAction.getPerform().length() > 0 ){
							//we go an action causing a perform
							Relationship relationship = new Relationship();
							relationship.setType(relationship.TYPE_PERFORM);
							relationship.setClientAction(currentAction);
							relationship.setClientActivityDiagram(currentActivityDiagram);
							
							String performedSubflow = currentAction.getPerform();
							
							int m;
							for (m = 0; m < activityDiagrams.size(); m++) {
								ActivityDiagram subflowDiagram = ((ActivityDiagram)activityDiagrams.get(m));
								if ((subflowDiagram.getType().equalsIgnoreCase(subflowDiagram.TYPE_SUBFLOW)) &&
										subflowDiagram.getName().equalsIgnoreCase(performedSubflow)){
									//thats the performed subflow activity diagram we are looking for
									
									relationship.setSupplierActivityDiagram(subflowDiagram);
									relationships.add(relationship);
								}
							}
							
						}
						
						

					}
					
				}
				
				
				
			} catch (NullPointerException e){
				
			}
		}
		
	}

	public void createIncludeRelationships() {
		
		int i;	
		for (i = 0; i < useCases.size(); i++){
			try {
				UseCase currentUseCase = ((UseCase)useCases.get(i));
				ArrayList activityDiagrams = currentUseCase.getActivityDiagrams();
				
				int j;
				for (j = 0 ; j < activityDiagrams.size(); j++){
					
					ActivityDiagram currentActivityDiagram = (ActivityDiagram)activityDiagrams.get(j);
					ArrayList actions = ((ArrayList)currentActivityDiagram.getActions());

					int k;
					for (k = 0 ; k < actions.size(); k++){
						
						Action currentAction = ((Action)actions.get(k)) ;
						
						if (currentAction.getInclude().length() > 0){
							
							String includedUseCaseName = currentAction.getInclude();
							UseCase includedUseCase = getUseCaseFromString (includedUseCaseName);
							
							Relationship relationship = new Relationship();
							relationship.setType(relationship.TYPE_INCLUDE);
							relationship.setClientUseCase(currentUseCase);
							relationship.setSupplierUseCase(includedUseCase);
							relationships.add(relationship);
							
						}
					}
				}
				
			} catch (NullPointerException e){
				
			}
			
			
		}
	}

	

	private void createAssociationRelationship(Actor actor, UseCase currentUseCase) {
		
		//first we must check that the association does not exist
		if (!isAssociationAlreadyThere(actor,currentUseCase)){
			Relationship relationship = new Relationship();
			relationship.setType(relationship.TYPE_ASSOCIATION);
			relationship.setActorAssociation(actor);
			relationship.setUseCaseAssociation(currentUseCase);
			relationships.add(relationship);
		}
		
	}

	private boolean isAssociationAlreadyThere(Actor actor, UseCase currentUseCase) {
		
		int i;
		for (i = 0; i < relationships.size(); i++){
			Relationship currentRelationship = ((Relationship)relationships.get(i));
			
			if (currentRelationship.getType().equalsIgnoreCase(currentRelationship.TYPE_ASSOCIATION)){
				
				if (currentRelationship.getActorAssociation().equals(actor) && 
						currentRelationship.getUseCaseAssociation().equals(currentUseCase)){
					return true;
				}
				
			}
		}
		
		return false;
	}

	private Actor getActorFromString(String actorName) {
		
		int i;
		for (i = 0; i < actors.size(); i++){
			Actor currentActor = ((Actor)actors.get(i));
			
			if (currentActor.getName().equalsIgnoreCase(actorName)){
				return currentActor;
			}
		}
		
		return null; //could not find the actor
	}
	
	public void createAssociationRelationships() {
		
		int i;	
		for (i = 0; i < useCases.size(); i++){
			
			try {
				
				UseCase currentUseCase = ((UseCase)useCases.get(i));
				ArrayList activityDiagrams = currentUseCase.getActivityDiagrams();
				
				int j;
				for (j = 0 ; j < activityDiagrams.size(); j++){
					
					ActivityDiagram currentActivityDiagram = (ActivityDiagram)activityDiagrams.get(j);
					ArrayList actions = ((ArrayList)currentActivityDiagram.getActions());
					
					int k;
					for (k = 0 ; k < actions.size(); k++){
						
						Action currentAction = ((Action)actions.get(k)) ;
						
						if (currentAction.getActor().length() > 0 && !currentAction.getActor().equalsIgnoreCase("SYSTEM")){
							String actorName = currentAction.getActor();
							Actor actor = getActorFromString(actorName);
							createAssociationRelationship(actor, currentUseCase);
						}
						
					}
				}
				
			} catch (NullPointerException e){
				
			}
			
			
		}
		
	}

	public void createGeneralizationRelationships() {
		int i;	
		for (i = 0; i < useCases.size(); i++){
			
			try {
				UseCase currentUseCase = ((UseCase)useCases.get(i));
				NameSection currentNameSection = currentUseCase.getNameSection();
				
				if (currentNameSection.isSpecializing()){
					
					String specializedUseCaseName = currentNameSection.getSpecializesUseCase();
					UseCase specializedUseCase = getUseCaseFromString (specializedUseCaseName);
					
					createGeneralizationRelationShipForUseCases(currentUseCase, specializedUseCase);
					
				}
				
				if (currentNameSection.isImplementing()){
					String implementedUseCaseName = currentNameSection.getImplementsUsecase();
					UseCase implementedUseCase = getUseCaseFromString (implementedUseCaseName);
					
					createGeneralizationRelationShipForUseCases(currentUseCase, implementedUseCase);
				}
				
			}catch (NullPointerException e){
				
			}
		}
		
		for (i = 0; i < actors.size(); i++){
			
			Actor currentActor = ((Actor)actors.get(i));
			
			if (currentActor.isSpecializing()){
				
				String specializedActorName = currentActor.getSpecializesActor();
				Actor specializedActor = getActorFromString (specializedActorName);
				
				createGeneralizationRelationShipForActors(currentActor, specializedActor);
				
			}
			
		}
			
		
	}

	private void createGeneralizationRelationShipForUseCases(UseCase childUseCase, UseCase parentUseCase) {
		
		//first we see if the generalization relationship already exists between these two use cases
		
		if (!isgeneralizationAlreadyThereForUseCases(childUseCase, parentUseCase)){
			Relationship relationship = new Relationship();
			relationship.setType(relationship.TYPE_GENERALIZATION);
			relationship.setClientUseCase(childUseCase);
			relationship.setSupplierUseCase(parentUseCase);
			relationships.add(relationship);
		}
		
	}
	


	private boolean isgeneralizationAlreadyThereForUseCases(UseCase childUseCase, UseCase parentUseCase) {
		int i;
		for (i = 0; i < relationships.size(); i++){
			Relationship currentRelationship = ((Relationship)relationships.get(i));
			
			if (currentRelationship.getType().equalsIgnoreCase(currentRelationship.TYPE_GENERALIZATION)){
				
				if (currentRelationship.getClientUseCase().equals(childUseCase) && 
						currentRelationship.getSupplierUseCase().equals(parentUseCase)){
					return true;
				}
				
			}
		}
		
		return false; //this generalization relationship does not already exist
	}
	
	private void createGeneralizationRelationShipForActors (Actor childActor, Actor parentActor) {
		
		//first we see if the generalization relationship already exists between these two use cases
		
		if (!isgeneralizationAlreadyThereForActors(childActor, parentActor)){
			Relationship relationship = new Relationship();
			relationship.setType(relationship.TYPE_GENERALIZATION_ACTOR);
			relationship.setChildActor(childActor);
			relationship.setParentActor(parentActor);
			relationships.add(relationship);
		}
		
	}
	
	private boolean isgeneralizationAlreadyThereForActors(Actor childActor, Actor parentActor) {
		int i;
		for (i = 0; i < relationships.size(); i++){
			Relationship currentRelationship = ((Relationship)relationships.get(i));
			
			if (currentRelationship.getType().equalsIgnoreCase(currentRelationship.TYPE_GENERALIZATION_ACTOR)){
				
				if (currentRelationship.getChildActor().equals(childActor) && 
						currentRelationship.getParentActor().equals(parentActor)){
					return true;
				}
				
			}
		}
		
		return false; //this generalization relationship does not already exist
	}

	public void createUseCaseDiagram(REUCD importedFrame) {
		
		//create the xml type objects first
		for (int i = 0; i < useCases.size(); i++){
			//create use case xml nodes
			UseCase currentUseCase = ((UseCase)useCases.get(i));
			
			UseCaseXMLNode newNode = new UseCaseXMLNode();
			newNode.setName(((NameSection)currentUseCase.getNameSection()).getName());
			newNode.setType(useCaseType);
			newNode.setId(useCaseIdPrefix + useCaseCounter);
			newNode.setUseCaseWrapperIdPrefix(newNode.getUseCaseWrapperIdPrefix() + useCaseCounter);
			useCaseCounter++;
			
			newNode.setAbstract(currentUseCase.getNameSection().isAbstract());

			useCaseXMLNodesList.add(newNode);
		}
		
		//here we set the includes, extends, extension points and generalizations of use cases
		for (int i = 0; i < useCases.size(); i++){
						
			UseCase currentUseCase = ((UseCase)useCases.get(i));
			//add all the extension points for a use case
			//remember that extension points don't have wrapper id
			ExtensionPointSection EPSection = currentUseCase.getExtensionPointsSection();
			
			try{
				ArrayList extensionPoints = (ArrayList)EPSection.getExtensionPoints();
				
				for (int j = 0 ; j < extensionPoints.size(); j++){
					
					ExtensionPoint currentExtensionPoint = ((ExtensionPoint)extensionPoints.get(j));
					
					if (currentExtensionPoint.getExtensionPointType().equalsIgnoreCase(currentExtensionPoint.TYPE_publicDeclaration)){
						
						ExtensionPointXMLNode newExtensionPointNode = new ExtensionPointXMLNode(); 
						
						newExtensionPointNode.setType(extensionPointType);
						newExtensionPointNode.setName(currentExtensionPoint.getExtensionPointName());
						newExtensionPointNode.setId(extensionPointIdPrefix + extensionPointCounter);
						extensionPointCounter++;
						
						for (int k =0 ; k < useCaseXMLNodesList.size(); k++ ){
							UseCaseXMLNode currentUseCaseXMLNode = ((UseCaseXMLNode)useCaseXMLNodesList.get(k));
							
							if (currentUseCaseXMLNode.getName().equalsIgnoreCase(currentUseCase.getNameSection().getName())){
								currentUseCaseXMLNode.getExtendPointList().add(newExtensionPointNode);
							}
							
						}
						
					}
				}
			} catch (NullPointerException e){
				
			}
			
			
			
			//check the includes
			for (int j = 0 ; j < includeList.size(); j++){
				Relationship currentInclude = ((Relationship)includeList.get(j));
				
				if (currentInclude.getClientUseCase().getNameSection().getName()
					.equalsIgnoreCase(currentUseCase.getNameSection().getName())){
					
					IncludeXMLNode newIncludeNode = new IncludeXMLNode();
					
					newIncludeNode.setType(includeType);
					newIncludeNode.setId(includeIdPrefix + includeCounter);
					newIncludeNode.setIncludeWrapperIdPrefix(newIncludeNode.getIncludeWrapperIdPrefix()+includeCounter);
					includeCounter++;
					
					UseCase additionUseCase = currentInclude.getSupplierUseCase();
					
					for (int k =0 ; k < useCaseXMLNodesList.size(); k++ ){
						UseCaseXMLNode currentUseCaseXMLNode = ((UseCaseXMLNode)useCaseXMLNodesList.get(k));
						
						if (currentUseCaseXMLNode.getName().equalsIgnoreCase(additionUseCase.getNameSection().getName())){
							newIncludeNode.setAdditon(currentUseCaseXMLNode.getId());
						}
						
						
					}
					
					for (int m =0 ; m < useCaseXMLNodesList.size(); m++ ){
						UseCaseXMLNode currentUseCaseXMLNode = ((UseCaseXMLNode)useCaseXMLNodesList.get(m));
						
						if (currentUseCaseXMLNode.getName().equalsIgnoreCase(currentUseCase.getNameSection().getName())){
							currentUseCaseXMLNode.getIncludeList().add(newIncludeNode);
						}
						
					}
		
					
				}
			}
			
			//check the extends
			for (int j = 0 ; j < extendList.size(); j++){
				Relationship currentExtend = ((Relationship)extendList.get(j));
				
				if (currentExtend.getClientUseCase().getNameSection().getName()
					.equalsIgnoreCase(currentUseCase.getNameSection().getName())){
					
					ExtendXMLNode newExtendNode = new ExtendXMLNode();
					
					
					
					newExtendNode.setType(extendType);
					newExtendNode.setId(extendIdPrefix + extendCounter);
					newExtendNode.setExtendWrapperIdPrefix(newExtendNode.getExtendWrapperIdPrefix()+extendCounter);
					newExtendNode.setCondition("["+currentExtend.getCondition()+"]");
					newExtendNode.setConditionWrapperId(conditionIdPrefix + conditionCounter);
					newExtendNode.setSpecificationWrapperId(specificationIdPrefix + specificationCounter);
					specificationCounter++;
					conditionCounter++;
					extendCounter++;
					
					UseCase extendedUseCase = currentExtend.getSupplierUseCase();
					
					for (int k =0 ; k < useCaseXMLNodesList.size(); k++ ){
						UseCaseXMLNode currentUseCaseXMLNode = ((UseCaseXMLNode)useCaseXMLNodesList.get(k));
						
						if (currentUseCaseXMLNode.getName().equalsIgnoreCase(extendedUseCase.getNameSection().getName())){
							newExtendNode.setExtendedUseCase(currentUseCaseXMLNode.getId());
							
							if (!currentExtend.getExtensionLocation().equalsIgnoreCase("")){
								
								ArrayList extensionPointsList = ((ArrayList)currentUseCaseXMLNode.getExtendPointList());
								for (int n = 0 ; n < extensionPointsList.size(); n++){
									
									ExtensionPointXMLNode currentExtensionPointXMLNode = ((ExtensionPointXMLNode)extensionPointsList.get(n));
									
									if (currentExtensionPointXMLNode.getName().equalsIgnoreCase(currentExtend.getExtensionLocation())){
										newExtendNode.setExtensionLocation(currentExtensionPointXMLNode.getId());

										currentExtensionPointXMLNode.getExtension().add(newExtendNode.getId());
									}
								}
							}
						}
						
						
						
						
					}
					
					for (int m =0 ; m < useCaseXMLNodesList.size(); m++ ){
						UseCaseXMLNode currentUseCaseXMLNode = ((UseCaseXMLNode)useCaseXMLNodesList.get(m));
						
						if (currentUseCaseXMLNode.getName().equalsIgnoreCase(currentUseCase.getNameSection().getName())){
							currentUseCaseXMLNode.getExtendList().add(newExtendNode);
						}
						
					}
					
				}
			}
			
			//check use case generalizations
			for (int j = 0 ; j < useCaseGeneralizationList.size(); j++){
				
				Relationship currentGeneralization = ((Relationship)useCaseGeneralizationList.get(j));
				
				if (currentGeneralization.getClientUseCase().getNameSection().getName()
						.equalsIgnoreCase(currentUseCase.getNameSection().getName())){
					
					GeneralizationXMLNode newGeneralizationNode= new GeneralizationXMLNode();
					
					newGeneralizationNode.setType(generalizationType);
					newGeneralizationNode.setId(generalizationIdPrefix + generalizationCounter);
					newGeneralizationNode.setGeneralizationWrapperIdPrefix(newGeneralizationNode.getGeneralizationWrapperIdPrefix()+generalizationCounter);
					generalizationCounter++;
					
					
					UseCase generalUseCase = currentGeneralization.getSupplierUseCase();
					
					for (int k =0 ; k < useCaseXMLNodesList.size(); k++ ){
						UseCaseXMLNode currentUseCaseXMLNode = ((UseCaseXMLNode)useCaseXMLNodesList.get(k));
						
						if (currentUseCaseXMLNode.getName().equalsIgnoreCase(generalUseCase.getNameSection().getName())){
							newGeneralizationNode.setGeneral(currentUseCaseXMLNode.getId());
						}
						
						
					}
					
					for (int m =0 ; m < useCaseXMLNodesList.size(); m++ ){
						UseCaseXMLNode currentUseCaseXMLNode = ((UseCaseXMLNode)useCaseXMLNodesList.get(m));
						
						if (currentUseCaseXMLNode.getName().equalsIgnoreCase(currentUseCase.getNameSection().getName())){
							currentUseCaseXMLNode.getGeneralizationList().add(newGeneralizationNode);
						}
						
					}

					
				}
				
				
			}
			
			
			
		}
		
		for (int i = 0; i < actors.size(); i++){
			//create actor xml nodes
			Actor currentActor = ((Actor)actors.get(i));
			
			ActorXMLNode newNode = new ActorXMLNode ();
			
			newNode.setName(currentActor.getName());
			newNode.setType(actorType);
			newNode.setId(actorIdPrefix + actorCounter);
			newNode.setActorWrapperIdPrefix(newNode.getActorWrapperIdPrefix()+actorCounter);
			actorCounter++;
			
			newNode.setAbstract(currentActor.isAbstract());
			
			actorXMLNodesList.add(newNode);
		}
		

		for (int j = 0 ; j <actorXMLNodesList.size(); j++){
			
			ActorXMLNode currentActorXMLNode = ((ActorXMLNode)actorXMLNodesList.get(j));
			
			//check actor generalizations
			for (int k = 0 ; k < actorGeneralizationList.size(); k++){
				
				System.out.println("actor generalization list size: " + actorGeneralizationList.size());
				
				Relationship currentGeneralization = ((Relationship)actorGeneralizationList.get(k));
				
				if (currentGeneralization.getChildActor().getName()
						.equalsIgnoreCase(currentActorXMLNode.getName())){
					
					GeneralizationXMLNode newGeneralizationNode= new GeneralizationXMLNode();
					
					newGeneralizationNode.setType(generalizationType);
					newGeneralizationNode.setId(generalizationIdPrefix + generalizationCounter);
					newGeneralizationNode.setGeneralizationWrapperIdPrefix(newGeneralizationNode.getGeneralizationWrapperIdPrefix()+generalizationCounter);
					generalizationCounter++;
					
					Actor generalActor = currentGeneralization.getParentActor();
					
					for (int m =0 ; m < actorXMLNodesList.size(); m++ ){
						ActorXMLNode currentActorXMLNode2 = ((ActorXMLNode)actorXMLNodesList.get(m));
						
						if (currentActorXMLNode2.getName().equalsIgnoreCase(generalActor.getName())){
							newGeneralizationNode.setGeneral(currentActorXMLNode2.getId());
						}
						
						
					}
					
					
					currentActorXMLNode.getGeneralizationList().add(newGeneralizationNode);
					
					
				}
				
			}
			
			
		}

		
		//here we set the generalizations for actors
		
		
		for (int i = 0; i < associationList.size(); i++){
			//create association nodes
			Relationship association = ((Relationship)associationList.get(i));
			
			AssociationXMLNode newNode = new AssociationXMLNode();
			
			newNode.setType(associationType);
			newNode.setId(associationIdPrefix + associationCounter);
			newNode.setAssociationWrapperidPrefix(newNode.getAssociationWrapperidPrefix()+associationCounter);
			associationCounter++;
			
			newNode.setActorEnd(associationActorEndIdPrefix + associationActorEndCounter);
			newNode.setActorRoleWrapperIdPrefix(newNode.getActorRoleWrapperIdPrefix() + associationActorEndCounter);
			associationActorEndCounter++;
			newNode.setUseCaseRoleWrapperIdPrefix(newNode.getUseCaseRoleWrapperIdPrefix()+associationUseCaseEndCounter);
			newNode.setUseCaseEnd(associationUseCaseEndIdPrefix + associationUseCaseEndCounter);
			
			associationUseCaseEndCounter++;
			
			UseCase useCaseEnd = association.getUseCaseAssociation();
			
			for (int j = 0 ; j < useCaseXMLNodesList.size(); j++){
				
				UseCaseXMLNode currentUseCaseXMLNode = ((UseCaseXMLNode)useCaseXMLNodesList.get(j));
				
				if (currentUseCaseXMLNode.getName().equalsIgnoreCase(((NameSection)useCaseEnd.getNameSection()).getName())){
					newNode.setUseCaseNode(currentUseCaseXMLNode);
				}
				
			}
			
			Actor actorEnd = association.getActorAssociation();
			
			for (int j = 0 ; j <actorXMLNodesList.size(); j++){
				
				ActorXMLNode currentActorXMLNode = ((ActorXMLNode)actorXMLNodesList.get(j));
				
				if (currentActorXMLNode.getName().equalsIgnoreCase(actorEnd.getName())){
					newNode.setActorNode(currentActorXMLNode);
				}
				
			}
			
			associationXMLNodesList.add(newNode);

		}
		
		//then pick up the xml file
		File currentDir = new File("C:\\REUCD");
		
		JFileChooser chooser = new JFileChooser(currentDir);
		chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
		
		ExampleFileFilter filter = new ExampleFileFilter();
		filter.addExtension("xml");
		filter.setDescription("Use Case XML File");
		chooser.setFileFilter(filter);
		chooser.setDialogTitle("Create Use Case XML File");
		
		int returnVal = chooser.showOpenDialog(importedFrame);
		if(returnVal != JFileChooser.APPROVE_OPTION) {
			return;
		}
		
		String fileName = chooser.getSelectedFile().getPath();
		
		try 
		{  
			
			DOMParser parser = new DOMParser();	  
			parser.parse("EmptyUCD.xml");
			doc = parser.getDocument();
			nonStaticDoc = doc;
			
		}catch (Exception e1) {	
			System.err.println("XML reading error: " + e1);
		}
		
		createUseCaseDiagram(nonStaticDoc);
		
		generateXMLString(nonStaticDoc);
		
		
		if (!fileName.endsWith("xml")){
			fileName = fileName + ".xml";
		}
		
		try {
			
			BufferedWriter out = new BufferedWriter(new FileWriter(fileName));
			out.write(getXMLString());
			out.close();
		} catch (IOException e2) {
		}
		
	}
	
	private void createUseCaseDiagram(Node node) {
		
		//time to write up the xml nodes
		int type = node.getNodeType();
		switch (type)
		{
		case Node.DOCUMENT_NODE: 
		{
			createUseCaseDiagram(((Document)node).getDocumentElement());
			break;
		}
		
		/**  element with attributes */
		case Node.ELEMENT_NODE: 
		{

			if (node.getNodeName().equalsIgnoreCase("uml:Model")) {

				
				for (int i = 0 ; i <actorXMLNodesList.size(); i++){
					
					ActorXMLNode newNode = ((ActorXMLNode)actorXMLNodesList.get(i));
					
					Element newElement;
					newElement = (node.getOwnerDocument()).createElement("ownedMember");
					
					newElement.setAttribute("name", newNode.getName());
					newElement.setAttribute("visibility","public");
					newElement.setAttribute("xmi:id", newNode.getId());
					newElement.setAttribute("xmi:type",newNode.getType());
					
					if(newNode.isAbstract()){
						newElement.setAttribute("isAbstract", "true");
					}
					
					ArrayList generalizationList = newNode.getGeneralizationList();
					
					for (int j = 0; j < generalizationList.size(); j++){
						GeneralizationXMLNode currentGeneralizationNode = ((GeneralizationXMLNode)generalizationList.get(j));
						
						Element generalizationElement;
						generalizationElement = (node.getOwnerDocument()).createElement("generalization");
						
						generalizationElement.setAttribute("general", currentGeneralizationNode.getGeneral());
						generalizationElement.setAttribute("xmi:id", currentGeneralizationNode.getId());
						generalizationElement.setAttribute("xmi:type",currentGeneralizationNode.getType());
						
						newElement.appendChild(generalizationElement);
						
					}
					
					node.appendChild(newElement);
					
				}
				
				for (int i = 0 ; i <useCaseXMLNodesList.size(); i++){
					
					UseCaseXMLNode newNode = ((UseCaseXMLNode)useCaseXMLNodesList.get(i));
					
					Element newElement;
					newElement = (node.getOwnerDocument()).createElement("ownedMember");
					
					newElement.setAttribute("name", newNode.getName());
					newElement.setAttribute("visibility","public");
					newElement.setAttribute("xmi:id", newNode.getId());
					newElement.setAttribute("xmi:type",newNode.getType());
					
					if(newNode.isAbstract()){
						newElement.setAttribute("isAbstract", "true");
					}
					
					ArrayList generalizationList = newNode.getGeneralizationList();
					
					for (int j = 0; j < generalizationList.size(); j++){
						GeneralizationXMLNode currentGeneralizationNode = ((GeneralizationXMLNode)generalizationList.get(j));
						
						Element generalizationElement;
						generalizationElement = (node.getOwnerDocument()).createElement("generalization");
						
						generalizationElement.setAttribute("general", currentGeneralizationNode.getGeneral());
						generalizationElement.setAttribute("xmi:id", currentGeneralizationNode.getId());
						generalizationElement.setAttribute("xmi:type",currentGeneralizationNode.getType());
						
						newElement.appendChild(generalizationElement);
						
					}
					
					ArrayList includeList = newNode.getIncludeList();
					
					for (int j = 0; j < includeList.size(); j++){
						IncludeXMLNode currentIncludeNode = ((IncludeXMLNode)includeList.get(j));
						
						Element includeElement;
						includeElement = (node.getOwnerDocument()).createElement("include");
						
						includeElement.setAttribute("addition", currentIncludeNode.getAdditon());
						includeElement.setAttribute("xmi:id", currentIncludeNode.getId());
						includeElement.setAttribute("xmi:type",currentIncludeNode.getType());
						includeElement.setAttribute("visibility","public");
						
						newElement.appendChild(includeElement);
						
					}
					
					ArrayList extendList = newNode.getExtendList();
					
					for (int j = 0; j < extendList.size(); j++){
						ExtendXMLNode currentExtendNode = ((ExtendXMLNode)extendList.get(j));
						
						Element extendElement;
						extendElement = (node.getOwnerDocument()).createElement("extend");
						
						extendElement.setAttribute("extendedCase", currentExtendNode.getExtendedUseCase());
						extendElement.setAttribute("xmi:id", currentExtendNode.getId());
						extendElement.setAttribute("xmi:type",currentExtendNode.getType());
						extendElement.setAttribute("visibility","public");
						
						if (!currentExtendNode.getExtensionLocation().equalsIgnoreCase("")){
							extendElement.setAttribute("extensionLocation",currentExtendNode.getExtensionLocation());
						}
						
						
						
						Element conditionElement;
						conditionElement = (node.getOwnerDocument()).createElement("condition");
						
						conditionElement.setAttribute("name", "unnamed1");
						conditionElement.setAttribute("xmi:id", currentExtendNode.getConditionWrapperId());
						conditionElement.setAttribute("xmi:type",conditionType);
						conditionElement.setAttribute("visibility","public");
						
						extendElement.appendChild(conditionElement);
						
						
						Element specificationElement;
						specificationElement = (node.getOwnerDocument()).createElement("specification");
						
						specificationElement.setAttribute("xmi:id", currentExtendNode.getSpecificationWrapperId());
						specificationElement.setAttribute("xmi:type",specificationType);
						specificationElement.setAttribute("visibility","public");
						specificationElement.setAttribute("language","English");
						specificationElement.setAttribute("body",currentExtendNode.getCondition());
						
						conditionElement.appendChild(specificationElement);
						
						newElement.appendChild(extendElement);
						
					}
					
					ArrayList extensionPointList = newNode.getExtendPointList();
					
					for (int j = 0; j < extensionPointList.size(); j++){
						ExtensionPointXMLNode currentExtensionPointNode = ((ExtensionPointXMLNode)extensionPointList.get(j));
						
						Element extensionPointElement;
						extensionPointElement = (node.getOwnerDocument()).createElement("extensionPoint");
						
						extensionPointElement.setAttribute("name", currentExtensionPointNode.getName());
						extensionPointElement.setAttribute("xmi:id", currentExtensionPointNode.getId());
						extensionPointElement.setAttribute("xmi:type",currentExtensionPointNode.getType());
						extensionPointElement.setAttribute("visibility","public");
						
						ArrayList extensionsList = currentExtensionPointNode.getExtension();
						
						for (int k = 0; k < extensionsList.size(); k++){
							
							
							Element extensionElement;
							extensionElement = (node.getOwnerDocument()).createElement("extension");
							
							String extensionID = ((String)extensionsList.get(k));
							extensionElement.setAttribute("xmi:idref",extensionID);

						}

						newElement.appendChild(extensionPointElement);
						
					}
					
					
					node.appendChild(newElement);
					
				}
				
				
				
				for (int i = 0 ; i <associationXMLNodesList.size(); i++){
					
					AssociationXMLNode newNode = ((AssociationXMLNode)associationXMLNodesList.get(i));
					
					Element newElement;
					newElement = (node.getOwnerDocument()).createElement("ownedMember");
					
					newElement.setAttribute("visibility","public");
					newElement.setAttribute("xmi:id", newNode.getId());
					newElement.setAttribute("xmi:type",newNode.getType());
					
					Element actorMemberElement;
					actorMemberElement = (node.getOwnerDocument()).createElement("memberEnd");
					
					actorMemberElement.setAttribute("xmi:idref",newNode.getActorEnd());
					newElement.appendChild(actorMemberElement);
					
					
					Element useCaseMemberElement;
					useCaseMemberElement = (node.getOwnerDocument()).createElement("memberEnd");
					
					useCaseMemberElement.setAttribute("xmi:idref",newNode.getUseCaseEnd());
					newElement.appendChild(useCaseMemberElement);
					
					
					
					Element actorOwnedElement;
					actorOwnedElement = (node.getOwnerDocument()).createElement("ownedEnd");
					
					actorOwnedElement.setAttribute("visibility","public");
					actorOwnedElement.setAttribute("xmi:id", newNode.getActorEnd());
					actorOwnedElement.setAttribute("xmi:type","uml:Property");
					actorOwnedElement.setAttribute("association", newNode.getId());
					actorOwnedElement.setAttribute("type",newNode.getActorNode().getId());
					
					newElement.appendChild(actorOwnedElement);
					
					
					Element useCaseOwnedElement;
					useCaseOwnedElement = (node.getOwnerDocument()).createElement("ownedEnd");
					
					useCaseOwnedElement.setAttribute("visibility","public");
					useCaseOwnedElement.setAttribute("xmi:id", newNode.getUseCaseEnd());
					useCaseOwnedElement.setAttribute("xmi:type","uml:Property");
					useCaseOwnedElement.setAttribute("association", newNode.getId());
					useCaseOwnedElement.setAttribute("type",newNode.getUseCaseNode().getId());
					
					newElement.appendChild(useCaseOwnedElement);
					
					
					
					node.appendChild(newElement);
				}
				
			}
			
			
			/****************************************
			 * 
			 * 
			 * time to write the wrapper xml nodes
			 * 
			 * 
			 * **************************************
			 */			
			
			if (node.getNodeName().equalsIgnoreCase("mdOwnedViews")) {
				
				Node parnetNode = node.getParentNode();
				NamedNodeMap attrs = parnetNode.getAttributes(); 
				Node attr = null;
				
				for (int i = 0; i < attrs.getLength(); i++)
				{
					attr = attrs.item(i);
					if (attr.getNodeName().equalsIgnoreCase("elementClass")) {
						break;
					} 
				}

				
				//starting with actors
				
				String attrType = attr.getNodeValue();
				if ( attrType.equalsIgnoreCase("DiagramPresentationElement")){
					
					for (int i = 0 ; i <actorXMLNodesList.size(); i++){
						ActorXMLNode newNode = ((ActorXMLNode)actorXMLNodesList.get(i));
						
						Element newElement;
						newElement = (node.getOwnerDocument()).createElement("mdElement");
						
						newElement.setAttribute("elementClass", "Actor");
						newElement.setAttribute("xmi:id", newNode.getActorWrapperIdPrefix());
						
						Element elementIDElement;
						elementIDElement = (node.getOwnerDocument()).createElement("elementID");
						elementIDElement.setAttribute("xmi:idref", newNode.getId());
						newElement.appendChild(elementIDElement);
						
						Element geometryElement;
						geometryElement = (node.getOwnerDocument()).createElement("geometry");
						geometryElement.setTextContent("200," + actorLayoutSeparator + ", 20, 20");
						actorLayoutSeparator = actorLayoutSeparator + seperationDistance;
						newElement.appendChild(geometryElement);
						
						node.appendChild(newElement);

					}
					
					
					
					for (int i = 0 ; i <actorXMLNodesList.size(); i++){
						ActorXMLNode newNode = ((ActorXMLNode)actorXMLNodesList.get(i));
						

						
						ArrayList generalizationList = newNode.getGeneralizationList();
						
						//the include wrapper are done with in each use case
						for (int j = 0; j < generalizationList.size(); j++){
							GeneralizationXMLNode currentGeneralizationNode = ((GeneralizationXMLNode)generalizationList.get(j));
							
							Element generalizationElement;
							generalizationElement = (node.getOwnerDocument()).createElement("mdElement");
							
							generalizationElement.setAttribute("elementClass", "Generalization");
							generalizationElement.setAttribute("xmi:id", currentGeneralizationNode.getGeneralizationWrapperIdPrefix());
							
							Element generalizationElementIDElement;
							generalizationElementIDElement = (node.getOwnerDocument()).createElement("elementID");
							generalizationElementIDElement.setAttribute("xmi:idref", currentGeneralizationNode.getId());
							generalizationElement.appendChild(generalizationElementIDElement);
							
							Element linkFirstEndIDElement;
							linkFirstEndIDElement = (node.getOwnerDocument()).createElement("linkFirstEndID");
							linkFirstEndIDElement.setAttribute("xmi:idref", getWrapper(currentGeneralizationNode.getGeneral()));
							generalizationElement.appendChild(linkFirstEndIDElement);
							
							Element linkSecondEndIDElement;
							linkSecondEndIDElement = (node.getOwnerDocument()).createElement("linkSecondEndID");
							linkSecondEndIDElement.setAttribute("xmi:idref", getWrapper(newNode.getId()));
							generalizationElement.appendChild(linkSecondEndIDElement);
							
							Element nameIDElement;
							nameIDElement = (node.getOwnerDocument()).createElement("linkNameID");
							nameIDElement.setAttribute("xmi:idref", textBoxIdPrefix +  textBoxCounter);
							generalizationElement.appendChild(nameIDElement);
							
							Element nameVisibleElement;
							nameVisibleElement = (node.getOwnerDocument()).createElement("nameVisible");
							nameVisibleElement.setAttribute("xmi:value", "true");
							generalizationElement.appendChild(nameVisibleElement);
							
							Element mdOwnedViewsElement;
							mdOwnedViewsElement = (node.getOwnerDocument()).createElement("mdOwnedViews");
							generalizationElement.appendChild(mdOwnedViewsElement);
							
							Element textBoxElement;
							textBoxElement = (node.getOwnerDocument()).createElement("mdElement");
							textBoxElement.setAttribute("xmi:id", textBoxIdPrefix +  textBoxCounter);
							textBoxElement.setAttribute("elementClass", "TextBox");
							mdOwnedViewsElement.appendChild(textBoxElement);
							
							textBoxCounter++;
							
							Element textBoxElement2;
							textBoxElement2 = (node.getOwnerDocument()).createElement("mdElement");
							textBoxElement2.setAttribute("xmi:id", textBoxIdPrefix +  textBoxCounter);
							textBoxElement2.setAttribute("elementClass", "TextBox");
							mdOwnedViewsElement.appendChild(textBoxElement2);
							
							textBoxCounter++;
							
							Element textBoxElement3;
							textBoxElement3 = (node.getOwnerDocument()).createElement("mdElement");
							textBoxElement3.setAttribute("xmi:id", textBoxIdPrefix +  textBoxCounter);
							textBoxElement3.setAttribute("elementClass", "TextBoxWithIcon");
							mdOwnedViewsElement.appendChild(textBoxElement3);
							
							textBoxCounter++;
							
							Element textBoxElement4;
							textBoxElement4 = (node.getOwnerDocument()).createElement("mdElement");
							textBoxElement4.setAttribute("xmi:id", textBoxIdPrefix +  textBoxCounter);
							textBoxElement4.setAttribute("elementClass", "TextBox");
							mdOwnedViewsElement.appendChild(textBoxElement4);
							
							Element linkConstraintIDElement;
							linkConstraintIDElement = (node.getOwnerDocument()).createElement("linkConstraintID");
							linkConstraintIDElement.setAttribute("xmi:idref", textBoxIdPrefix +  (textBoxCounter-2));
							generalizationElement.appendChild(linkConstraintIDElement);
							
							Element linkStereotypeIDElement;
							linkStereotypeIDElement = (node.getOwnerDocument()).createElement("linkStereotypeID");
							linkStereotypeIDElement.setAttribute("xmi:idref", textBoxIdPrefix +  (textBoxCounter-1));
							generalizationElement.appendChild(linkStereotypeIDElement);
							
							Element linktaggedValuesIDElement;
							linktaggedValuesIDElement = (node.getOwnerDocument()).createElement("linktaggedValuesID");
							linktaggedValuesIDElement.setAttribute("xmi:idref", textBoxIdPrefix +  textBoxCounter);
							generalizationElement.appendChild(linktaggedValuesIDElement);
							
							textBoxCounter++;
							
							node.appendChild(generalizationElement);
						}
						
					}
					
					
					//and now the use case wrapper ndoes
					for (int i = 0 ; i <useCaseXMLNodesList.size(); i++){
						UseCaseXMLNode newNode = ((UseCaseXMLNode)useCaseXMLNodesList.get(i));
						
						Element newElement;
						newElement = (node.getOwnerDocument()).createElement("mdElement");
						
						newElement.setAttribute("elementClass", "UseCase");
						newElement.setAttribute("xmi:id", newNode.getUseCaseWrapperIdPrefix());
						
						Element elementIDElement;
						elementIDElement = (node.getOwnerDocument()).createElement("elementID");
						elementIDElement.setAttribute("xmi:idref", newNode.getId());
						newElement.appendChild(elementIDElement);
						
						Element geometryElement;
						geometryElement = (node.getOwnerDocument()).createElement("geometry");
						geometryElement.setTextContent("500," + useCaseLayoutSeparator + ", 20, 20");
						useCaseLayoutSeparator = useCaseLayoutSeparator + seperationDistance;
						newElement.appendChild(geometryElement);
						
						node.appendChild(newElement);

						
					}
					
					
					
					for (int i = 0 ; i <useCaseXMLNodesList.size(); i++){
						UseCaseXMLNode newNode = ((UseCaseXMLNode)useCaseXMLNodesList.get(i));
						
						ArrayList includeList = newNode.getIncludeList();
						
						//the include wrapper are done with in each use case
						for (int j = 0; j < includeList.size(); j++){
							IncludeXMLNode currentIncludeNode = ((IncludeXMLNode)includeList.get(j));
							
							Element includeElement;
							includeElement = (node.getOwnerDocument()).createElement("mdElement");
							
							includeElement.setAttribute("elementClass", "Include");
							includeElement.setAttribute("xmi:id", currentIncludeNode.getIncludeWrapperIdPrefix());
							
							Element includeElementIDElement;
							includeElementIDElement = (node.getOwnerDocument()).createElement("elementID");
							includeElementIDElement.setAttribute("xmi:idref", currentIncludeNode.getId());
							includeElement.appendChild(includeElementIDElement);
							
							Element linkFirstEndIDElement;
							linkFirstEndIDElement = (node.getOwnerDocument()).createElement("linkFirstEndID");
							linkFirstEndIDElement.setAttribute("xmi:idref", getWrapper(currentIncludeNode.getAdditon()));
							includeElement.appendChild(linkFirstEndIDElement);
							
							Element linkSecondEndIDElement;
							linkSecondEndIDElement = (node.getOwnerDocument()).createElement("linkSecondEndID");
							linkSecondEndIDElement.setAttribute("xmi:idref", newNode.getUseCaseWrapperIdPrefix());
							includeElement.appendChild(linkSecondEndIDElement);
							
							Element nameIDElement;
							nameIDElement = (node.getOwnerDocument()).createElement("linkNameID");
							nameIDElement.setAttribute("xmi:idref", textBoxIdPrefix +  textBoxCounter);
							includeElement.appendChild(nameIDElement);
							
							Element nameVisibleElement;
							nameVisibleElement = (node.getOwnerDocument()).createElement("nameVisible");
							nameVisibleElement.setAttribute("xmi:value", "true");
							includeElement.appendChild(nameVisibleElement);
							
							Element mdOwnedViewsElement;
							mdOwnedViewsElement = (node.getOwnerDocument()).createElement("mdOwnedViews");
							includeElement.appendChild(mdOwnedViewsElement);
							
							Element textBoxElement;
							textBoxElement = (node.getOwnerDocument()).createElement("mdElement");
							textBoxElement.setAttribute("xmi:id", textBoxIdPrefix +  textBoxCounter);
							textBoxElement.setAttribute("elementClass", "TextBox");
							mdOwnedViewsElement.appendChild(textBoxElement);
							
							textBoxCounter++;
							
							Element textBoxElement2;
							textBoxElement2 = (node.getOwnerDocument()).createElement("mdElement");
							textBoxElement2.setAttribute("xmi:id", textBoxIdPrefix +  textBoxCounter);
							textBoxElement2.setAttribute("elementClass", "TextBox");
							mdOwnedViewsElement.appendChild(textBoxElement2);
							
							textBoxCounter++;
							
							Element textBoxElement3;
							textBoxElement3 = (node.getOwnerDocument()).createElement("mdElement");
							textBoxElement3.setAttribute("xmi:id", textBoxIdPrefix +  textBoxCounter);
							textBoxElement3.setAttribute("elementClass", "TextBoxWithIcon");
							mdOwnedViewsElement.appendChild(textBoxElement3);
							
							Element includeTextElement;
							includeTextElement = (node.getOwnerDocument()).createElement("text");
							includeTextElement.setTextContent("<<include>>");
							textBoxElement3.appendChild(includeTextElement);
							
							textBoxCounter++;
							
							Element textBoxElement4;
							textBoxElement4 = (node.getOwnerDocument()).createElement("mdElement");
							textBoxElement4.setAttribute("xmi:id", textBoxIdPrefix +  textBoxCounter);
							textBoxElement4.setAttribute("elementClass", "TextBox");
							mdOwnedViewsElement.appendChild(textBoxElement4);
							
							Element linkConstraintIDElement;
							linkConstraintIDElement = (node.getOwnerDocument()).createElement("linkConstraintID");
							linkConstraintIDElement.setAttribute("xmi:idref", textBoxIdPrefix +  (textBoxCounter-2));
							includeElement.appendChild(linkConstraintIDElement);
							
							Element linkStereotypeIDElement;
							linkStereotypeIDElement = (node.getOwnerDocument()).createElement("linkStereotypeID");
							linkStereotypeIDElement.setAttribute("xmi:idref", textBoxIdPrefix +  (textBoxCounter-1));
							includeElement.appendChild(linkStereotypeIDElement);
							
							Element linktaggedValuesIDElement;
							linktaggedValuesIDElement = (node.getOwnerDocument()).createElement("linktaggedValuesID");
							linktaggedValuesIDElement.setAttribute("xmi:idref", textBoxIdPrefix +  textBoxCounter);
							includeElement.appendChild(linktaggedValuesIDElement);
							
							textBoxCounter++;
							
							node.appendChild(includeElement);
							
						}

					}
					
					
					for (int i = 0 ; i <useCaseXMLNodesList.size(); i++){
						UseCaseXMLNode newNode = ((UseCaseXMLNode)useCaseXMLNodesList.get(i));
						
						ArrayList generalizationList = newNode.getGeneralizationList();
						
						//the include wrapper are done with in each use case
						for (int j = 0; j < generalizationList.size(); j++){
							GeneralizationXMLNode currentGeneralizationNode = ((GeneralizationXMLNode)generalizationList.get(j));
							
							Element generalizationElement;
							generalizationElement = (node.getOwnerDocument()).createElement("mdElement");
							
							generalizationElement.setAttribute("elementClass", "Generalization");
							generalizationElement.setAttribute("xmi:id", currentGeneralizationNode.getGeneralizationWrapperIdPrefix());
							
							Element generalizationElementIDElement;
							generalizationElementIDElement = (node.getOwnerDocument()).createElement("elementID");
							generalizationElementIDElement.setAttribute("xmi:idref", currentGeneralizationNode.getId());
							generalizationElement.appendChild(generalizationElementIDElement);
							
							Element linkFirstEndIDElement;
							linkFirstEndIDElement = (node.getOwnerDocument()).createElement("linkFirstEndID");
							linkFirstEndIDElement.setAttribute("xmi:idref", getWrapper(currentGeneralizationNode.getGeneral()));
							generalizationElement.appendChild(linkFirstEndIDElement);
							
							Element linkSecondEndIDElement;
							linkSecondEndIDElement = (node.getOwnerDocument()).createElement("linkSecondEndID");
							linkSecondEndIDElement.setAttribute("xmi:idref", getWrapper(newNode.getId()));
							generalizationElement.appendChild(linkSecondEndIDElement);
							
							Element nameIDElement;
							nameIDElement = (node.getOwnerDocument()).createElement("linkNameID");
							nameIDElement.setAttribute("xmi:idref", textBoxIdPrefix +  textBoxCounter);
							generalizationElement.appendChild(nameIDElement);
							
							Element nameVisibleElement;
							nameVisibleElement = (node.getOwnerDocument()).createElement("nameVisible");
							nameVisibleElement.setAttribute("xmi:value", "true");
							generalizationElement.appendChild(nameVisibleElement);
							
							Element mdOwnedViewsElement;
							mdOwnedViewsElement = (node.getOwnerDocument()).createElement("mdOwnedViews");
							generalizationElement.appendChild(mdOwnedViewsElement);
							
							Element textBoxElement;
							textBoxElement = (node.getOwnerDocument()).createElement("mdElement");
							textBoxElement.setAttribute("xmi:id", textBoxIdPrefix +  textBoxCounter);
							textBoxElement.setAttribute("elementClass", "TextBox");
							mdOwnedViewsElement.appendChild(textBoxElement);
							
							textBoxCounter++;
							
							Element textBoxElement2;
							textBoxElement2 = (node.getOwnerDocument()).createElement("mdElement");
							textBoxElement2.setAttribute("xmi:id", textBoxIdPrefix +  textBoxCounter);
							textBoxElement2.setAttribute("elementClass", "TextBox");
							mdOwnedViewsElement.appendChild(textBoxElement2);
							
							textBoxCounter++;
							
							Element textBoxElement3;
							textBoxElement3 = (node.getOwnerDocument()).createElement("mdElement");
							textBoxElement3.setAttribute("xmi:id", textBoxIdPrefix +  textBoxCounter);
							textBoxElement3.setAttribute("elementClass", "TextBoxWithIcon");
							mdOwnedViewsElement.appendChild(textBoxElement3);
							
							textBoxCounter++;
							
							Element textBoxElement4;
							textBoxElement4 = (node.getOwnerDocument()).createElement("mdElement");
							textBoxElement4.setAttribute("xmi:id", textBoxIdPrefix +  textBoxCounter);
							textBoxElement4.setAttribute("elementClass", "TextBox");
							mdOwnedViewsElement.appendChild(textBoxElement4);
							
							Element linkConstraintIDElement;
							linkConstraintIDElement = (node.getOwnerDocument()).createElement("linkConstraintID");
							linkConstraintIDElement.setAttribute("xmi:idref", textBoxIdPrefix +  (textBoxCounter-2));
							generalizationElement.appendChild(linkConstraintIDElement);
							
							Element linkStereotypeIDElement;
							linkStereotypeIDElement = (node.getOwnerDocument()).createElement("linkStereotypeID");
							linkStereotypeIDElement.setAttribute("xmi:idref", textBoxIdPrefix +  (textBoxCounter-1));
							generalizationElement.appendChild(linkStereotypeIDElement);
							
							Element linktaggedValuesIDElement;
							linktaggedValuesIDElement = (node.getOwnerDocument()).createElement("linktaggedValuesID");
							linktaggedValuesIDElement.setAttribute("xmi:idref", textBoxIdPrefix +  textBoxCounter);
							generalizationElement.appendChild(linktaggedValuesIDElement);
							
							textBoxCounter++;
							
							node.appendChild(generalizationElement);
						}
						
					}
					
					
					for (int i = 0 ; i <useCaseXMLNodesList.size(); i++){
						UseCaseXMLNode newNode = ((UseCaseXMLNode)useCaseXMLNodesList.get(i));
						
						
						ArrayList extendList = newNode.getExtendList();
						
						//the extend wrapper are done with in each use case
						for (int j = 0; j < extendList.size(); j++){
							ExtendXMLNode currentExtendNode = ((ExtendXMLNode)extendList.get(j));
							
							Element extendElement;
							extendElement = (node.getOwnerDocument()).createElement("mdElement");
							
							extendElement.setAttribute("elementClass", "Extend");
							extendElement.setAttribute("xmi:id", currentExtendNode.getExtendWrapperIdPrefix());
							
							Element extendElementIDElement;
							extendElementIDElement = (node.getOwnerDocument()).createElement("elementID");
							extendElementIDElement.setAttribute("xmi:idref", currentExtendNode.getId());
							extendElement.appendChild(extendElementIDElement);
							
							Element linkFirstEndIDElement;
							linkFirstEndIDElement = (node.getOwnerDocument()).createElement("linkFirstEndID");
							linkFirstEndIDElement.setAttribute("xmi:idref", getWrapper(currentExtendNode.getExtendedUseCase()));
							extendElement.appendChild(linkFirstEndIDElement);
							
							Element linkSecondEndIDElement;
							linkSecondEndIDElement = (node.getOwnerDocument()).createElement("linkSecondEndID");
							linkSecondEndIDElement.setAttribute("xmi:idref", newNode.getUseCaseWrapperIdPrefix());
							extendElement.appendChild(linkSecondEndIDElement);
							
							Element nameIDElement;
							nameIDElement = (node.getOwnerDocument()).createElement("linkNameID");
							nameIDElement.setAttribute("xmi:idref", textBoxIdPrefix +  textBoxCounter);
							extendElement.appendChild(nameIDElement);
							
							Element nameVisibleElement;
							nameVisibleElement = (node.getOwnerDocument()).createElement("nameVisible");
							nameVisibleElement.setAttribute("xmi:value", "true");
							extendElement.appendChild(nameVisibleElement);
							
							Element mdOwnedViewsElement;
							mdOwnedViewsElement = (node.getOwnerDocument()).createElement("mdOwnedViews");
							extendElement.appendChild(mdOwnedViewsElement);
							
							
							Element textBoxElement;
							textBoxElement = (node.getOwnerDocument()).createElement("mdElement");
							textBoxElement.setAttribute("xmi:id", textBoxIdPrefix +  textBoxCounter);
							textBoxElement.setAttribute("elementClass", "TextBox");
							mdOwnedViewsElement.appendChild(textBoxElement);
							
							textBoxCounter++;
							
							Element textBoxElement2;
							textBoxElement2 = (node.getOwnerDocument()).createElement("mdElement");
							textBoxElement2.setAttribute("xmi:id", textBoxIdPrefix +  textBoxCounter);
							textBoxElement2.setAttribute("elementClass", "TextBox");
							mdOwnedViewsElement.appendChild(textBoxElement2);
							
							textBoxCounter++;
							
							Element textBoxElement3;
							textBoxElement3 = (node.getOwnerDocument()).createElement("mdElement");
							textBoxElement3.setAttribute("xmi:id", textBoxIdPrefix +  textBoxCounter);
							textBoxElement3.setAttribute("elementClass", "TextBoxWithIcon");
							mdOwnedViewsElement.appendChild(textBoxElement3);
							
							Element extendTextElement;
							extendTextElement = (node.getOwnerDocument()).createElement("text");
							extendTextElement.setTextContent("<<extend>>");
							textBoxElement3.appendChild(extendTextElement);
							
							textBoxCounter++;
							
							Element textBoxElement4;
							textBoxElement4 = (node.getOwnerDocument()).createElement("mdElement");
							textBoxElement4.setAttribute("xmi:id", textBoxIdPrefix +  textBoxCounter);
							textBoxElement4.setAttribute("elementClass", "TextBox");
							mdOwnedViewsElement.appendChild(textBoxElement4);
							
							textBoxCounter++;
							
							Element textBoxElement5;
							textBoxElement5 = (node.getOwnerDocument()).createElement("mdElement");
							textBoxElement5.setAttribute("xmi:id", textBoxIdPrefix +  textBoxCounter);
							textBoxElement5.setAttribute("elementClass", "TextBox");
							mdOwnedViewsElement.appendChild(textBoxElement5);
							
							//we'll make the first text box be the one that represents the extension location
							if (!currentExtendNode.getExtensionLocation().equalsIgnoreCase("")){									
								
								Element typeLocationTextElement;
								typeLocationTextElement = (node.getOwnerDocument()).createElement("text");
								typeLocationTextElement.setTextContent(currentExtendNode.getExtensionLocation());
								textBoxElement5.appendChild(typeLocationTextElement);

							}
							
							Element linkConstraintIDElement;
							linkConstraintIDElement = (node.getOwnerDocument()).createElement("linkConstraintID");
							linkConstraintIDElement.setAttribute("xmi:idref", textBoxIdPrefix +  (textBoxCounter-3));
							extendElement.appendChild(linkConstraintIDElement);
							
							Element linkStereotypeIDElement;
							linkStereotypeIDElement = (node.getOwnerDocument()).createElement("linkStereotypeID");
							linkStereotypeIDElement.setAttribute("xmi:idref", textBoxIdPrefix +  (textBoxCounter-2));
							extendElement.appendChild(linkStereotypeIDElement);
							
							Element linktaggedValuesIDElement;
							linktaggedValuesIDElement = (node.getOwnerDocument()).createElement("linktaggedValuesID");
							linktaggedValuesIDElement.setAttribute("xmi:idref", textBoxIdPrefix +  (textBoxCounter-1));
							extendElement.appendChild(linktaggedValuesIDElement);
							
							Element extensionPointIDElement;
							extensionPointIDElement = (node.getOwnerDocument()).createElement("extensionPointID");
							extensionPointIDElement.setAttribute("xmi:idref", textBoxIdPrefix +  textBoxCounter);
							extendElement.appendChild(extensionPointIDElement);
						
							
							textBoxCounter++;
							
							node.appendChild(extendElement);
							
						}
						
					}
					
					
					
					
					
					
					//Do the ASSOCIATION WRAPPER XML NODES
					
					System.out.println("association list size: " + associationXMLNodesList.size());
					for (int i = 0 ; i <associationXMLNodesList.size(); i++){
						
						AssociationXMLNode newNode = ((AssociationXMLNode)associationXMLNodesList.get(i));
						
						Element newElement;
						newElement = (node.getOwnerDocument()).createElement("mdElement");
						
						newElement.setAttribute("elementClass", "Association");
						newElement.setAttribute("xmi:id", newNode.getAssociationWrapperidPrefix());
						
						Element elementIDElement;
						elementIDElement = (node.getOwnerDocument()).createElement("elementID");
						elementIDElement.setAttribute("xmi:idref", newNode.getId());
						newElement.appendChild(elementIDElement);
						
						Element linkFirstEndIDElement;
						linkFirstEndIDElement = (node.getOwnerDocument()).createElement("linkFirstEndID");
						linkFirstEndIDElement.setAttribute("xmi:idref",newNode.getUseCaseNode().getUseCaseWrapperIdPrefix());
						newElement.appendChild(linkFirstEndIDElement);
						
						Element linkSecondEndIDElement;
						linkSecondEndIDElement = (node.getOwnerDocument()).createElement("linkSecondEndID");
						linkSecondEndIDElement.setAttribute("xmi:idref", newNode.getActorNode().getActorWrapperIdPrefix());
						newElement.appendChild(linkSecondEndIDElement);
						
						Element nameIDElement;
						nameIDElement = (node.getOwnerDocument()).createElement("linkNameID");
						nameIDElement.setAttribute("xmi:idref", textBoxIdPrefix +  textBoxCounter);
						newElement.appendChild(nameIDElement);
						
						Element nameVisibleElement;
						nameVisibleElement = (node.getOwnerDocument()).createElement("nameVisible");
						nameVisibleElement.setAttribute("xmi:value", "true");
						newElement.appendChild(nameVisibleElement);
						
						
						Element mdOwnedViewsElement;
						mdOwnedViewsElement = (node.getOwnerDocument()).createElement("mdOwnedViews");
						newElement.appendChild(mdOwnedViewsElement);
						
						Element associationTextBoxElement;
						associationTextBoxElement = (node.getOwnerDocument()).createElement("mdElement");
						associationTextBoxElement.setAttribute("elementClass", "AssociationTextBox");
						associationTextBoxElement.setAttribute("xmi:id", textBoxIdPrefix +  (textBoxCounter));
						mdOwnedViewsElement.appendChild(associationTextBoxElement);
						
						textBoxCounter++;
						
						
						Element textBoxElement;
						textBoxElement = (node.getOwnerDocument()).createElement("mdElement");
						textBoxElement.setAttribute("xmi:id", textBoxIdPrefix +  textBoxCounter);
						textBoxElement.setAttribute("elementClass", "TextBox");
						mdOwnedViewsElement.appendChild(textBoxElement);
						
						textBoxCounter++;
						
						Element textBoxElement2;
						textBoxElement2 = (node.getOwnerDocument()).createElement("mdElement");
						textBoxElement2.setAttribute("xmi:id", textBoxIdPrefix +  textBoxCounter);
						textBoxElement2.setAttribute("elementClass", "TextBoxWithIcon");
						mdOwnedViewsElement.appendChild(textBoxElement2);
						
						textBoxCounter++;
						
						Element textBoxElement3;
						textBoxElement3 = (node.getOwnerDocument()).createElement("mdElement");
						textBoxElement3.setAttribute("xmi:id", textBoxIdPrefix +  textBoxCounter);
						textBoxElement3.setAttribute("elementClass", "TextBox");
						mdOwnedViewsElement.appendChild(textBoxElement3);
						
						
						Element linkConstraintIDElement;
						linkConstraintIDElement = (node.getOwnerDocument()).createElement("linkConstraintID");
						linkConstraintIDElement.setAttribute("xmi:idref", textBoxIdPrefix +  (textBoxCounter-2));
						newElement.appendChild(linkConstraintIDElement);
						
						Element linkStereotypeIDElement;
						linkStereotypeIDElement = (node.getOwnerDocument()).createElement("linkStereotypeID");
						linkStereotypeIDElement.setAttribute("xmi:idref", textBoxIdPrefix +  (textBoxCounter-1));
						newElement.appendChild(linkStereotypeIDElement);
						
						Element linkTaggedValuesIDElement;
						linkTaggedValuesIDElement = (node.getOwnerDocument()).createElement("linkTaggedValuesID");
						linkTaggedValuesIDElement.setAttribute("xmi:idref", textBoxIdPrefix +  (textBoxCounter));
						newElement.appendChild(linkTaggedValuesIDElement);
						
						textBoxCounter++;
						
						Element associationFirstEndIDElement;
						associationFirstEndIDElement = (node.getOwnerDocument()).createElement("associationFirstEndID");
						associationFirstEndIDElement.setAttribute("xmi:idref",newNode.getUseCaseRoleWrapperIdPrefix());
						newElement.appendChild(associationFirstEndIDElement);
						
						Element associationSecondEndIDElement;
						associationSecondEndIDElement = (node.getOwnerDocument()).createElement("associationSecondEndID");
						associationSecondEndIDElement.setAttribute("xmi:idref", newNode.getActorRoleWrapperIdPrefix());
						newElement.appendChild(associationSecondEndIDElement);

						
						
						
						Element actorRoleElement;
						actorRoleElement = (node.getOwnerDocument()).createElement("mdElement");
						actorRoleElement.setAttribute("elementClass", "Role");
						actorRoleElement.setAttribute("xmi:id", newNode.getActorRoleWrapperIdPrefix());
						mdOwnedViewsElement.appendChild(actorRoleElement);
						
						Element actorElementIDRoleElement;
						actorElementIDRoleElement = (node.getOwnerDocument()).createElement("elementID");
						actorElementIDRoleElement.setAttribute("xmi:idref", newNode.getActorEnd());
						actorRoleElement.appendChild(actorElementIDRoleElement);
						
						Element mdActorRoleOwnedViewsElement;
						mdActorRoleOwnedViewsElement = (node.getOwnerDocument()).createElement("mdOwnedViews");
						actorRoleElement.appendChild(mdActorRoleOwnedViewsElement);
						
						
						
						Element textBoxElementActorRole;
						textBoxElementActorRole = (node.getOwnerDocument()).createElement("mdElement");
						textBoxElementActorRole.setAttribute("xmi:id", textBoxIdPrefix +  textBoxCounter);
						textBoxElementActorRole.setAttribute("elementClass", "TextBox");
						mdActorRoleOwnedViewsElement.appendChild(textBoxElementActorRole);
						
						textBoxCounter++;
						
						Element textBoxElementActorRole2;
						textBoxElementActorRole2 = (node.getOwnerDocument()).createElement("mdElement");
						textBoxElementActorRole2.setAttribute("xmi:id", textBoxIdPrefix +  textBoxCounter);
						textBoxElementActorRole2.setAttribute("elementClass", "TextBox");
						mdActorRoleOwnedViewsElement.appendChild(textBoxElementActorRole2);
						
						textBoxCounter++;
						
						Element textBoxElementActorRole3;
						textBoxElementActorRole3 = (node.getOwnerDocument()).createElement("mdElement");
						textBoxElementActorRole3.setAttribute("xmi:id", textBoxIdPrefix +  textBoxCounter);
						textBoxElementActorRole3.setAttribute("elementClass", "TextBox");
						mdActorRoleOwnedViewsElement.appendChild(textBoxElementActorRole3);
						
						textBoxCounter++;
						
						Element textBoxElementActorRole4;
						textBoxElementActorRole4 = (node.getOwnerDocument()).createElement("mdElement");
						textBoxElementActorRole4.setAttribute("xmi:id", textBoxIdPrefix +  textBoxCounter);
						textBoxElementActorRole4.setAttribute("elementClass", "TextBox");
						mdActorRoleOwnedViewsElement.appendChild(textBoxElementActorRole4);
						
						textBoxCounter++;
						
						Element textBoxElementActorRole5;
						textBoxElementActorRole5 = (node.getOwnerDocument()).createElement("mdElement");
						textBoxElementActorRole5.setAttribute("xmi:id", textBoxIdPrefix +  textBoxCounter);
						textBoxElementActorRole5.setAttribute("elementClass", "TextBoxWithIcon");
						mdActorRoleOwnedViewsElement.appendChild(textBoxElementActorRole5);
						
						textBoxCounter++;
						
						Element textBoxElementActorRole6;
						textBoxElementActorRole6 = (node.getOwnerDocument()).createElement("mdElement");
						textBoxElementActorRole6.setAttribute("xmi:id", textBoxIdPrefix +  textBoxCounter);
						textBoxElementActorRole6.setAttribute("elementClass", "TextBox");
						mdActorRoleOwnedViewsElement.appendChild(textBoxElementActorRole6);
						
						
						Element roleNameIDActorElement;
						roleNameIDActorElement = (node.getOwnerDocument()).createElement("roleNameID");
						roleNameIDActorElement.setAttribute("xmi:idref", textBoxIdPrefix +  (textBoxCounter-5));
						actorRoleElement.appendChild(roleNameIDActorElement);
						
						Element roleMultiplicityIDActorElement;
						roleMultiplicityIDActorElement = (node.getOwnerDocument()).createElement("roleMultiplicityID");
						roleMultiplicityIDActorElement.setAttribute("xmi:idref", textBoxIdPrefix +  (textBoxCounter-4));
						actorRoleElement.appendChild(roleMultiplicityIDActorElement);
						
						Element rolePropertyStringIDActorElement;
						rolePropertyStringIDActorElement = (node.getOwnerDocument()).createElement("rolePropertyStringID");
						rolePropertyStringIDActorElement.setAttribute("xmi:idref", textBoxIdPrefix +  (textBoxCounter-3));
						actorRoleElement.appendChild(rolePropertyStringIDActorElement);
						
						Element roleConstraintsIDActorElement;
						roleConstraintsIDActorElement = (node.getOwnerDocument()).createElement("roleConstraintsID");
						roleConstraintsIDActorElement.setAttribute("xmi:idref", textBoxIdPrefix +  (textBoxCounter-2));
						actorRoleElement.appendChild(roleConstraintsIDActorElement);
						
						Element roleStereotypeIDActorElement;
						roleStereotypeIDActorElement = (node.getOwnerDocument()).createElement("roleStereotypeID");
						roleStereotypeIDActorElement.setAttribute("xmi:idref", textBoxIdPrefix +  (textBoxCounter-1));
						actorRoleElement.appendChild(roleStereotypeIDActorElement);
						
						Element roleTaggedValuesIDActorElement;
						roleTaggedValuesIDActorElement = (node.getOwnerDocument()).createElement("roleTaggedValuesID");
						roleTaggedValuesIDActorElement.setAttribute("xmi:idref", textBoxIdPrefix +  (textBoxCounter));
						actorRoleElement.appendChild(roleTaggedValuesIDActorElement);
						
						textBoxCounter++;
						
						
						
						
						
						Element useCaseRoleElement;
						useCaseRoleElement = (node.getOwnerDocument()).createElement("mdElement");
						useCaseRoleElement.setAttribute("elementClass", "Role");
						useCaseRoleElement.setAttribute("xmi:id", newNode.getUseCaseRoleWrapperIdPrefix());
						mdOwnedViewsElement.appendChild(useCaseRoleElement);
						
						Element useCaseElementIDRoleElement;
						useCaseElementIDRoleElement = (node.getOwnerDocument()).createElement("elementID");
						useCaseElementIDRoleElement.setAttribute("xmi:idref", newNode.getUseCaseEnd());
						useCaseRoleElement.appendChild(useCaseElementIDRoleElement);
						
						Element mdUseCaseRoleOwnedViewsElement;
						mdUseCaseRoleOwnedViewsElement = (node.getOwnerDocument()).createElement("mdOwnedViews");
						useCaseRoleElement.appendChild(mdUseCaseRoleOwnedViewsElement);
						
						
						
						Element textBoxElementUseCaseRole;
						textBoxElementUseCaseRole = (node.getOwnerDocument()).createElement("mdElement");
						textBoxElementUseCaseRole.setAttribute("xmi:id", textBoxIdPrefix +  textBoxCounter);
						textBoxElementUseCaseRole.setAttribute("elementClass", "TextBox");
						mdUseCaseRoleOwnedViewsElement.appendChild(textBoxElementUseCaseRole);
						
						textBoxCounter++;
						
						Element textBoxElementUseCaseRole2;
						textBoxElementUseCaseRole2 = (node.getOwnerDocument()).createElement("mdElement");
						textBoxElementUseCaseRole2.setAttribute("xmi:id", textBoxIdPrefix +  textBoxCounter);
						textBoxElementUseCaseRole2.setAttribute("elementClass", "TextBox");
						mdUseCaseRoleOwnedViewsElement.appendChild(textBoxElementUseCaseRole2);
						
						textBoxCounter++;
						
						Element textBoxElementUseCaseRole3;
						textBoxElementUseCaseRole3 = (node.getOwnerDocument()).createElement("mdElement");
						textBoxElementUseCaseRole3.setAttribute("xmi:id", textBoxIdPrefix +  textBoxCounter);
						textBoxElementUseCaseRole3.setAttribute("elementClass", "TextBox");
						mdUseCaseRoleOwnedViewsElement.appendChild(textBoxElementUseCaseRole3);
						
						textBoxCounter++;
						
						Element textBoxElementUseCaseRole4;
						textBoxElementUseCaseRole4 = (node.getOwnerDocument()).createElement("mdElement");
						textBoxElementUseCaseRole4.setAttribute("xmi:id", textBoxIdPrefix +  textBoxCounter);
						textBoxElementUseCaseRole4.setAttribute("elementClass", "TextBox");
						mdUseCaseRoleOwnedViewsElement.appendChild(textBoxElementUseCaseRole4);
						
						textBoxCounter++;
						
						Element textBoxElementUseCaseRole5;
						textBoxElementUseCaseRole5 = (node.getOwnerDocument()).createElement("mdElement");
						textBoxElementUseCaseRole5.setAttribute("xmi:id", textBoxIdPrefix +  textBoxCounter);
						textBoxElementUseCaseRole5.setAttribute("elementClass", "TextBoxWithIcon");
						mdUseCaseRoleOwnedViewsElement.appendChild(textBoxElementUseCaseRole5);
						
						textBoxCounter++;
						
						Element textBoxElementUseCaseRole6;
						textBoxElementUseCaseRole6 = (node.getOwnerDocument()).createElement("mdElement");
						textBoxElementUseCaseRole6.setAttribute("xmi:id", textBoxIdPrefix +  textBoxCounter);
						textBoxElementUseCaseRole6.setAttribute("elementClass", "TextBox");
						mdUseCaseRoleOwnedViewsElement.appendChild(textBoxElementUseCaseRole6);
						
						
						
						Element roleNameIDUseCaseElement;
						roleNameIDUseCaseElement = (node.getOwnerDocument()).createElement("roleNameID");
						roleNameIDUseCaseElement.setAttribute("xmi:idref", textBoxIdPrefix +  (textBoxCounter-5));
						useCaseRoleElement.appendChild(roleNameIDUseCaseElement);
						
						Element roleMultiplicityIDUseCaseElement;
						roleMultiplicityIDUseCaseElement = (node.getOwnerDocument()).createElement("roleMultiplicityID");
						roleMultiplicityIDUseCaseElement.setAttribute("xmi:idref", textBoxIdPrefix +  (textBoxCounter-4));
						useCaseRoleElement.appendChild(roleMultiplicityIDUseCaseElement);
						
						Element rolePropertyStringIDUseCaseElement;
						rolePropertyStringIDUseCaseElement = (node.getOwnerDocument()).createElement("rolePropertyStringID");
						rolePropertyStringIDUseCaseElement.setAttribute("xmi:idref", textBoxIdPrefix +  (textBoxCounter-3));
						useCaseRoleElement.appendChild(rolePropertyStringIDUseCaseElement);
						
						Element roleConstraintsIDUseCaseElement;
						roleConstraintsIDUseCaseElement = (node.getOwnerDocument()).createElement("roleConstraintsID");
						roleConstraintsIDUseCaseElement.setAttribute("xmi:idref", textBoxIdPrefix +  (textBoxCounter-2));
						useCaseRoleElement.appendChild(roleConstraintsIDUseCaseElement);
						
						Element roleStereotypeIDUseCaseElement;
						roleStereotypeIDUseCaseElement = (node.getOwnerDocument()).createElement("roleStereotypeID");
						roleStereotypeIDUseCaseElement.setAttribute("xmi:idref", textBoxIdPrefix +  (textBoxCounter-1));
						useCaseRoleElement.appendChild(roleStereotypeIDUseCaseElement);
						
						Element roleTaggedValuesIDUseCaseElement;
						roleTaggedValuesIDUseCaseElement = (node.getOwnerDocument()).createElement("roleTaggedValuesID");
						roleTaggedValuesIDUseCaseElement.setAttribute("xmi:idref", textBoxIdPrefix +  (textBoxCounter));
						useCaseRoleElement.appendChild(roleTaggedValuesIDUseCaseElement);
						
						textBoxCounter++;
						
						node.appendChild(newElement);
					}
	
					
				}
				
			}
			
			
			
			
			NodeList children = node.getChildNodes();
			
			if (children != null)
			{
				int len = children.getLength();
				for (int i = 0; i < len; i++)
					createUseCaseDiagram(children.item(i));
			}
			
			break;
			
		}
		}//end switch
		
		
	}

	public String getXMLString() {
		return xmlStr;
	}
	
	private void generateXMLString(Document doc) {
		
		StringWriter  strWriter    = null;
		XMLSerializer probeMsgSerializer   = null;
		OutputFormat  outFormat    = null;
		try {
			probeMsgSerializer = new XMLSerializer();
			strWriter = new StringWriter();
			outFormat = new OutputFormat();
			
			// Setup format settings
			outFormat.setIndenting(true);
			outFormat.setIndent(4);
			
			// Define a Writer
			probeMsgSerializer.setOutputCharStream(strWriter);
			
			// Apply the format settings
			probeMsgSerializer.setOutputFormat(outFormat);
			
			// Serialize XML Document
			probeMsgSerializer.serialize(doc);
			xmlStr = strWriter.toString();
			strWriter.close();
			
		} catch (IOException ioEx) {
			System.out.println("Error " + ioEx);
		}
	}
	
	
	private String getWrapper(String keyId) {
		
		for (int i = 0; i < useCaseXMLNodesList.size(); i++){
			UseCaseXMLNode newNode = ((UseCaseXMLNode)useCaseXMLNodesList.get(i));
			
			if (newNode.getId().equalsIgnoreCase(keyId)){
				return newNode.getUseCaseWrapperIdPrefix();
			}
		}
		
		for (int i = 0; i < actorXMLNodesList.size(); i++){
			ActorXMLNode newNode = ((ActorXMLNode)actorXMLNodesList.get(i));
			
			if (newNode.getId().equalsIgnoreCase(keyId)){
				return newNode.getActorWrapperIdPrefix();
			}
		}
		
		//should never get here
		return null;
	}

	public ArrayList getActorGeneralizationList() {
		return actorGeneralizationList;
	}

	public void setActorGeneralizationList(ArrayList actorGeneralizationList) {
		this.actorGeneralizationList = actorGeneralizationList;
	}

	public ArrayList getAssociationList() {
		return associationList;
	}

	public void setAssociationList(ArrayList associationList) {
		this.associationList = associationList;
	}

	public ArrayList getExtendList() {
		return extendList;
	}

	public void setExtendList(ArrayList extendList) {
		this.extendList = extendList;
	}

	public ArrayList getIncludeList() {
		return includeList;
	}

	public void setIncludeList(ArrayList includeList) {
		this.includeList = includeList;
	}

	public ArrayList getUseCaseGeneralizationList() {
		return useCaseGeneralizationList;
	}

	public void setUseCaseGeneralizationList(ArrayList useCaseGeneralizationList) {
		this.useCaseGeneralizationList = useCaseGeneralizationList;
	}

	public ArrayList getPerformList() {
		return performList;
	}

	public void setPerformList(ArrayList performList) {
		this.performList = performList;
	}
	

	
}